#!/usr/bin/env python
#
# Copyright (c) 2009 Mark Lee <libdesktop-agnostic@lazymalevolence.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.

import sys
import gobject
from desktopagnostic import vfs


def on_file_count_changed(trash, mainloop):
    print 'Number of files in the trash: %u' % trash.props.file_count
    mainloop.quit()


def main():
    vfs.init()
    try:
        trash = vfs.Trash.get_default()
        mainloop = gobject.MainLoop()
        trash.connect('file-count-changed', on_file_count_changed, mainloop)
        mainloop.run()
    finally:
        vfs.shutdown()
    return 0

if __name__ == '__main__':
    sys.exit(main())



#! /usr/bin/env python
# encoding: utf-8

import os.path,shutil
import Task,Runner,Utils,Logs,Build,Node
from TaskGen import extension,after,before
EXT_VALA=['.vala','.gs']
class valac_task(Task.Task):
	vars=("VALAC","VALAC_VERSION","VALAFLAGS")
	before=("cc","cxx")
	def run(self):
		env=self.env
		inputs=[a.srcpath(env)for a in self.inputs]
		valac=env['VALAC']
		vala_flags=env.get_flat('VALAFLAGS')
		top_src=self.generator.bld.srcnode.abspath()
		top_bld=self.generator.bld.srcnode.abspath(env)
		if env['VALAC_VERSION']>(0,1,6):
			cmd=[valac,'-C','--quiet',vala_flags]
		else:
			cmd=[valac,'-C',vala_flags]
		if self.threading:
			cmd.append('--thread')
		if self.target_glib:
			cmd.append('--target-glib=%s'%self.target_glib)
		features=self.generator.features
		if'cshlib'in features or'cstaticlib'in features:
			output_dir=self.outputs[0].bld_dir(env)
			cmd.append('--library '+self.target)
			if env['VALAC_VERSION']>=(0,7,0):
				header_basename=getattr(self,'header',self.target)
				cmd.append('--header '+os.path.join(output_dir,header_basename+'.h'))
				self.outputs.append(self.generator.path.find_or_declare(header_basename+'.h'))
				if env['VALAC_VERSION']>(0,7,5):
					includedir=self.generator.path.abspath().replace(top_src+'/','',1)
					cmd.append('--includedir '+includedir)
			cmd.append('--basedir '+top_src)
			cmd.append('-d '+top_bld)
			if env['VALAC_VERSION']>(0,7,2)and hasattr(self,'gir'):
				cmd.append('--gir=%s.gir'%self.gir)
		else:
			output_dir=self.outputs[0].bld_dir(env)
			cmd.append('-d %s'%output_dir)
		for vapi_dir in self.vapi_dirs:
			cmd.append('--vapidir=%s'%vapi_dir)
		for package in self.packages:
			cmd.append('--pkg %s'%package)
		for package in self.packages_private:
			cmd.append('--pkg %s'%package)
		cmd.append(" ".join(inputs))
		result=self.generator.bld.exec_command(" ".join(cmd))
		if not'cprogram'in features:
			if self.packages:
				filename=os.path.join(self.generator.path.abspath(env),"%s.deps"%self.target)
				deps=open(filename,'w')
				for package in self.packages:
					deps.write(package+'\n')
				deps.close()
			self._fix_output("../%s.vapi"%self.target)
			self._fix_output("%s.vapi"%self.target)
			self._fix_output("%s.gidl"%self.target)
			self._fix_output("%s.gir"%self.target)
			if hasattr(self,'gir'):
				self._fix_output("%s.gir"%self.gir)
		return result
	def install(self):
		bld=self.generator.bld
		features=self.generator.features
		if self.attr("install_path")and("cshlib"in features or"cstaticlib"in features):
			headers_list=[o for o in self.outputs if o.suffix()==".h"]
			vapi_list=[o for o in self.outputs if(o.suffix()in(".vapi",".deps"))]
			gir_list=[o for o in self.outputs if o.suffix()==".gir"]
			for header in headers_list:
				top_src=self.generator.bld.srcnode
				package=self.env['PACKAGE']
				try:
					api_version=Utils.g_module.API_VERSION
				except AttributeError:
					version=Utils.g_module.VERSION.split(".")
					if version[0]=="0":
						api_version="0."+version[1]
					else:
						api_version=version[0]+".0"
				install_path="${INCLUDEDIR}/%s-%s/%s"%(package,api_version,header.relpath_gen(top_src))
				bld.install_as(install_path,header.abspath(self.env),self.env)
			for vapi in vapi_list:
				bld.install_files("${DATAROOTDIR}/vala/vapi",vapi.abspath(self.env),self.env)
			for gir in gir_list:
				bld.install_files("${DATAROOTDIR}/gir-1.0",gir.abspath(self.env),self.env)
	def _fix_output(self,output):
		top_bld=self.generator.bld.srcnode.abspath(self.env)
		try:
			src=os.path.join(top_bld,output)
			dst=self.generator.path.abspath(self.env)
			shutil.move(src,dst)
		except:
			pass
def vala_file(self,node):
	valatask=getattr(self,"valatask",None)
	if not valatask:
		valatask=self.create_task('valac')
		self.valatask=valatask
		self.includes=Utils.to_list(getattr(self,'includes',[]))
		valatask.packages=[]
		valatask.packages_private=Utils.to_list(getattr(self,'packages_private',[]))
		valatask.vapi_dirs=[]
		valatask.target=self.target
		valatask.threading=False
		valatask.install_path=self.install_path
		valatask.target_glib=None
		packages=Utils.to_list(getattr(self,'packages',[]))
		vapi_dirs=Utils.to_list(getattr(self,'vapi_dirs',[]))
		includes=[]
		if hasattr(self,'uselib_local'):
			local_packages=Utils.to_list(self.uselib_local)
			seen=[]
			while len(local_packages)>0:
				package=local_packages.pop()
				if package in seen:
					continue
				seen.append(package)
				package_obj=self.name_to_obj(package)
				if not package_obj:
					raise Utils.WafError("object '%s' was not found in uselib_local (required by '%s')"%(package,self.name))
				package_name=package_obj.target
				package_node=package_obj.path
				package_dir=package_node.relpath_gen(self.path)
				for task in package_obj.tasks:
					for output in task.outputs:
						if output.name==package_name+".vapi":
							valatask.set_run_after(task)
							if package_name not in packages:
								packages.append(package_name)
							if package_dir not in vapi_dirs:
								vapi_dirs.append(package_dir)
							if package_dir not in includes:
								includes.append(package_dir)
				if hasattr(package_obj,'uselib_local'):
					lst=self.to_list(package_obj.uselib_local)
					lst.reverse()
					local_packages=[pkg for pkg in lst if pkg not in seen]+local_packages
		valatask.packages=packages
		for vapi_dir in vapi_dirs:
			try:
				valatask.vapi_dirs.append(self.path.find_dir(vapi_dir).abspath())
				valatask.vapi_dirs.append(self.path.find_dir(vapi_dir).abspath(self.env))
			except AttributeError:
				Logs.warn("Unable to locate Vala API directory: '%s'"%vapi_dir)
		self.includes.append(node.bld.srcnode.abspath())
		self.includes.append(node.bld.srcnode.abspath(self.env))
		for include in includes:
			try:
				self.includes.append(self.path.find_dir(include).abspath())
				self.includes.append(self.path.find_dir(include).abspath(self.env))
			except AttributeError:
				Logs.warn("Unable to locate include directory: '%s'"%include)
		if hasattr(self,'threading'):
			valatask.threading=self.threading
			self.uselib=self.to_list(self.uselib)
			if not'GTHREAD'in self.uselib:
				self.uselib.append('GTHREAD')
		if hasattr(self,'target_glib'):
			valatask.target_glib=self.target_glib
		if hasattr(self,'gir'):
			valatask.gir=self.gir
		if hasattr(self,'header'):
			valatask.header=self.header
	env=valatask.env
	output_nodes=[]
	c_node=node.change_ext('.c')
	output_nodes.append(c_node)
	self.allnodes.append(c_node)
	if env['VALAC_VERSION']<(0,7,0):
		output_nodes.append(node.change_ext('.h'))
	else:
		if not'cprogram'in self.features:
			output_nodes.append(self.path.find_or_declare('%s.h'%getattr(self,'header',self.target)))
	if not'cprogram'in self.features:
		output_nodes.append(self.path.find_or_declare('%s.vapi'%self.target))
		if env['VALAC_VERSION']>(0,7,2):
			if hasattr(self,'gir'):
				output_nodes.append(self.path.find_or_declare('%s.gir'%self.gir))
		elif env['VALAC_VERSION']>(0,3,5):
			output_nodes.append(self.path.find_or_declare('%s.gir'%self.target))
		elif env['VALAC_VERSION']>(0,1,7):
			output_nodes.append(self.path.find_or_declare('%s.gidl'%self.target))
		if valatask.packages:
			output_nodes.append(self.path.find_or_declare('%s.deps'%self.target))
	valatask.inputs.append(node)
	valatask.outputs.extend(output_nodes)
def detect(conf):
	min_version=(0,1,6)
	min_version_str="%d.%d.%d"%min_version
	valac=conf.find_program('valac',var='VALAC',mandatory=True)
	if not conf.env["HAVE_GTHREAD"]:
		conf.check_cfg(package='gthread-2.0',uselib_store='GTHREAD',args='--cflags --libs')
	try:
		output=Utils.cmd_output(valac+" --version",silent=True)
		version=output.split(' ',1)[-1].strip().split(".")
		version=[int(x)for x in version]
		valac_version=tuple(version)
	except Exception:
		valac_version=(0,0,0)
	conf.check_message('program version','valac >= '+min_version_str,valac_version>=min_version,"%d.%d.%d"%valac_version)
	conf.check_tool('gnu_dirs')
	if valac_version<min_version:
		conf.fatal("valac version too old to be used with this tool")
		return
	conf.env['VALAC_VERSION']=valac_version
	conf.env['VALAFLAGS']=''

extension(EXT_VALA)(vala_file)



#! /usr/bin/env python
# encoding: utf-8

import os,sys
import Configure,Options,Utils
import ccroot,ar,gxx
from Configure import conftest
def find_icpc(conf):
	if sys.platform=='cygwin':
		conf.fatal('The Intel compiler does not work on Cygwin')
	v=conf.env
	cxx=None
	if v['CXX']:cxx=v['CXX']
	elif'CXX'in conf.environ:cxx=conf.environ['CXX']
	if not cxx:cxx=conf.find_program('icpc',var='CXX')
	if not cxx:conf.fatal('Intel C++ Compiler (icpc) was not found')
	cxx=conf.cmd_to_list(cxx)
	ccroot.get_cc_version(conf,cxx,icc=True)
	v['CXX']=cxx
	v['CXX_NAME']='icc'
detect='''
find_icpc
find_ar
gxx_common_flags
gxx_modifier_darwin
cxx_load_tools
cxx_add_flags
'''

conftest(find_icpc)



#! /usr/bin/env python
# encoding: utf-8

import os,sys
import Configure,Options,Utils
import ccroot,ar
from Configure import conftest
def find_gcc(conf):
	v=conf.env
	cc=None
	if v['CC']:
		cc=v['CC']
	elif'CC'in conf.environ:
		cc=conf.environ['CC']
	if not cc:cc=conf.find_program('gcc',var='CC')
	if not cc:cc=conf.find_program('cc',var='CC')
	if not cc:conf.fatal('gcc was not found')
	cc=conf.cmd_to_list(cc)
	ccroot.get_cc_version(conf,cc,gcc=True)
	v['CC_NAME']='gcc'
	v['CC']=cc
def gcc_common_flags(conf):
	v=conf.env
	v['CC_SRC_F']=''
	v['CC_TGT_F']=['-c','-o','']
	v['CPPPATH_ST']='-I%s'
	if not v['LINK_CC']:v['LINK_CC']=v['CC']
	v['CCLNK_SRC_F']=''
	v['CCLNK_TGT_F']=['-o','']
	v['LIB_ST']='-l%s'
	v['LIBPATH_ST']='-L%s'
	v['STATICLIB_ST']='-l%s'
	v['STATICLIBPATH_ST']='-L%s'
	v['RPATH_ST']='-Wl,-rpath,%s'
	v['CCDEFINES_ST']='-D%s'
	v['SONAME_ST']='-Wl,-h,%s'
	v['SHLIB_MARKER']='-Wl,-Bdynamic'
	v['STATICLIB_MARKER']='-Wl,-Bstatic'
	v['FULLSTATIC_MARKER']='-static'
	v['program_PATTERN']='%s'
	v['shlib_CCFLAGS']=['-fPIC','-DPIC']
	v['shlib_LINKFLAGS']=['-shared']
	v['shlib_PATTERN']='lib%s.so'
	v['staticlib_LINKFLAGS']=['-Wl,-Bstatic']
	v['staticlib_PATTERN']='lib%s.a'
	v['LINKFLAGS_MACBUNDLE']=['-bundle','-undefined','dynamic_lookup']
	v['CCFLAGS_MACBUNDLE']=['-fPIC']
	v['macbundle_PATTERN']='%s.bundle'
def gcc_modifier_win32(conf):
	v=conf.env
	v['program_PATTERN']='%s.exe'
	v['shlib_PATTERN']='%s.dll'
	v['implib_PATTERN']='lib%s.dll.a'
	v['IMPLIB_ST']='-Wl,--out-implib,%s'
	v['shlib_CXXFLAGS']=[]
	v.append_value('LINKFLAGS','-Wl,--enable-auto-import')
def gcc_modifier_cygwin(conf):
	gxx_modifier_win32(conf)
	v=conf.env
	v['shlib_PATTERN']='cyg%s.dll'
def gcc_modifier_darwin(conf):
	v=conf.env
	v['shlib_CCFLAGS']=['-fPIC','-compatibility_version','1','-current_version','1']
	v['shlib_LINKFLAGS']=['-dynamiclib']
	v['shlib_PATTERN']='lib%s.dylib'
	v['staticlib_LINKFLAGS']=[]
	v['SHLIB_MARKER']=''
	v['STATICLIB_MARKER']=''
def gcc_modifier_aix5(conf):
	v=conf.env
	v['program_LINKFLAGS']=['-Wl,-brtl']
	v['shlib_LINKFLAGS']=['-shared','-Wl,-brtl,-bexpfull']
	v['SHLIB_MARKER']=''
def detect(conf):
	conf.find_gcc()
	conf.find_cpp()
	conf.find_ar()
	conf.gcc_common_flags()
	target_platform=conf.env['TARGET_PLATFORM']or sys.platform
	gcc_modifier_func=globals().get('gcc_modifier_'+target_platform)
	if gcc_modifier_func:
		gcc_modifier_func(conf)
	conf.cc_load_tools()
	conf.cc_add_flags()

conftest(find_gcc)
conftest(gcc_common_flags)
conftest(gcc_modifier_win32)
conftest(gcc_modifier_cygwin)
conftest(gcc_modifier_darwin)
conftest(gcc_modifier_aix5)



#! /usr/bin/env python
# encoding: utf-8

import os,sys,re
import TaskGen,Task,Utils,preproc,Logs,Build,Options
from Logs import error,debug,warn
from Utils import md5
from TaskGen import taskgen,after,before,feature
from Constants import*
try:
	from cStringIO import StringIO
except ImportError:
	from io import StringIO
import config_c
USE_TOP_LEVEL=False
win_platform=sys.platform in('win32','cygwin')
def get_cc_version(conf,cc,gcc=False,icc=False):
	cmd=cc+['-dM','-E','-']
	try:
		p=Utils.pproc.Popen(cmd,stdin=Utils.pproc.PIPE,stdout=Utils.pproc.PIPE,stderr=Utils.pproc.PIPE)
		p.stdin.write('\n')
		out=p.communicate()[0]
	except:
		conf.fatal('could not determine the compiler version %r'%cmd)
	out=str(out)
	if gcc:
		if out.find('__INTEL_COMPILER')>=0:
			conf.fatal('The intel compiler pretends to be gcc')
		if out.find('__GNUC__')<0:
			conf.fatal('Could not determine the compiler type')
	if icc and out.find('__INTEL_COMPILER')<0:
		conf.fatal('Not icc/icpc')
	k={}
	if icc or gcc:
		out=out.split('\n')
		import shlex
		for line in out:
			lst=shlex.split(line)
			if len(lst)>2:
				key=lst[1]
				val=lst[2]
				k[key]=val
		conf.env['CC_VERSION']=(k['__GNUC__'],k['__GNUC_MINOR__'],k['__GNUC_PATCHLEVEL__'])
	return k
class DEBUG_LEVELS:
	ULTRADEBUG="ultradebug"
	DEBUG="debug"
	RELEASE="release"
	OPTIMIZED="optimized"
	CUSTOM="custom"
	ALL=[ULTRADEBUG,DEBUG,RELEASE,OPTIMIZED,CUSTOM]
def scan(self):
	debug('ccroot: _scan_preprocessor(self, node, env, path_lst)')
	if len(self.inputs)==1:
		node=self.inputs[0]
		(nodes,names)=preproc.get_deps(node,self.env,nodepaths=self.env['INC_PATHS'])
		if Logs.verbose:
			debug('deps: deps for %s: %r; unresolved %r'%(str(node),nodes,names))
		return(nodes,names)
	all_nodes=[]
	all_names=[]
	seen=[]
	for node in self.inputs:
		(nodes,names)=preproc.get_deps(node,self.env,nodepaths=self.env['INC_PATHS'])
		if Logs.verbose:
			debug('deps: deps for %s: %r; unresolved %r'%(str(node),nodes,names))
		for x in nodes:
			if id(x)in seen:continue
			seen.append(id(x))
			all_nodes.append(x)
		for x in names:
			if not x in all_names:
				all_names.append(x)
	return(all_nodes,all_names)
class ccroot_abstract(TaskGen.task_gen):
	def __init__(self,*k,**kw):
		if len(k)>1:
			k=list(k)
			if k[1][0]!='c':
				k[1]='c'+k[1]
		TaskGen.task_gen.__init__(self,*k,**kw)
def get_target_name(self):
	tp='program'
	for x in self.features:
		if x in['cshlib','cstaticlib']:
			tp=x.lstrip('c')
	pattern=self.env[tp+'_PATTERN']
	if not pattern:pattern='%s'
	dir,name=os.path.split(self.target)
	if win_platform and getattr(self,'vnum','')and'cshlib'in self.features:
		name=name+'-'+self.vnum.split('.')[0]
	return os.path.join(dir,pattern%name)
def install_implib(self):
	bld=self.outputs[0].__class__.bld
	bindir=self.install_path
	if not len(self.outputs)==2:
		raise ValueError('fail')
	dll=self.outputs[0]
	bld.install_as(bindir+os.sep+dll.name,dll.abspath(self.env),chmod=self.chmod,env=self.env)
	implib=self.outputs[1]
	libdir='${LIBDIR}'
	if not self.env['LIBDIR']:
		libdir='${PREFIX}/lib'
	if sys.platform=='cygwin':
		bld.symlink_as(libdir+'/'+implib.name,bindir+os.sep+dll.name,env=self.env)
	else:
		bld.install_as(libdir+'/'+implib.name,implib.abspath(self.env),env=self.env)
def install_shlib(self):
	bld=self.outputs[0].__class__.bld
	nums=self.vnum.split('.')
	path=self.install_path
	if not path:return
	libname=self.outputs[0].name
	name3=libname+'.'+self.vnum
	name2=libname+'.'+nums[0]
	name1=libname
	filename=self.outputs[0].abspath(self.env)
	bld.install_as(os.path.join(path,name3),filename,env=self.env)
	bld.symlink_as(os.path.join(path,name2),name3)
	bld.symlink_as(os.path.join(path,name1),name3)
def default_cc(self):
	Utils.def_attrs(self,includes='',defines='',rpaths='',uselib='',uselib_local='',add_objects='',p_flag_vars=[],p_type_vars=[],compiled_tasks=[],link_task=None)
def apply_verif(self):
	if not(self.source or getattr(self,'add_objects',None)):
		raise Utils.WafError('no source files specified for %s'%self)
	if not self.target:
		raise Utils.WafError('no target for %s'%self)
def vars_target_cprogram(self):
	self.default_install_path=self.env['BINDIR']or'${PREFIX}/bin'
	self.default_chmod=O755
def vars_target_cstaticlib(self):
	self.default_install_path=self.env['LIBDIR']or'${PREFIX}/lib${LIB_EXT}'
def vars_target_cshlib(self):
	if win_platform:
		self.default_install_path=self.env['BINDIR']or'${PREFIX}/bin'
		self.default_chmod=O755
	else:
		self.default_install_path=self.env['LIBDIR']or'${PREFIX}/lib${LIB_EXT}'
def install_target_cstaticlib(self):
	if not self.bld.is_install:return
	self.link_task.install_path=self.install_path
def install_target_cshlib(self):
	if getattr(self,'vnum','')and not win_platform:
		self.link_task.vnum=self.vnum
		self.link_task.install=install_shlib
def apply_incpaths(self):
	lst=[]
	for lib in self.to_list(self.uselib):
		for path in self.env['CPPPATH_'+lib]:
			if not path in lst:
				lst.append(path)
	if preproc.go_absolute:
		for path in preproc.standard_includes:
			if not path in lst:
				lst.append(path)
	for path in self.to_list(self.includes):
		if not path in lst:
			if preproc.go_absolute or not os.path.isabs(path):
				lst.append(path)
			else:
				self.env.prepend_value('CPPPATH',path)
	for path in lst:
		node=None
		if os.path.isabs(path):
			if preproc.go_absolute:
				node=self.bld.root.find_dir(path)
		elif path[0]=='#':
			node=self.bld.srcnode
			if len(path)>1:
				node=node.find_dir(path[1:])
		else:
			node=self.path.find_dir(path)
		if node:
			self.env.append_value('INC_PATHS',node)
	if USE_TOP_LEVEL:
		self.env.append_value('INC_PATHS',self.bld.srcnode)
def apply_type_vars(self):
	for x in self.features:
		if not x in['cprogram','cstaticlib','cshlib']:
			continue
		x=x.lstrip('c')
		st=self.env[x+'_USELIB']
		if st:self.uselib=self.uselib+' '+st
		for var in self.p_type_vars:
			compvar='%s_%s'%(x,var)
			value=self.env[compvar]
			if value:self.env.append_value(var,value)
def apply_link(self):
	link=getattr(self,'link',None)
	if not link:
		if'cstaticlib'in self.features:link='ar_link_static'
		elif'cxx'in self.features:link='cxx_link'
		else:link='cc_link'
		if'cshlib'in self.features:
			if win_platform:
				link='dll_'+link
			elif getattr(self,'vnum',''):
				if sys.platform=='darwin':
					self.vnum=''
				else:
					link='vnum_'+link
	tsk=self.create_task(link)
	outputs=[t.outputs[0]for t in self.compiled_tasks]
	tsk.set_inputs(outputs)
	tsk.set_outputs(self.path.find_or_declare(get_target_name(self)))
	tsk.chmod=self.chmod
	self.link_task=tsk
def apply_lib_vars(self):
	env=self.env
	uselib=self.to_list(self.uselib)
	seen=[]
	names=self.to_list(self.uselib_local)[:]
	while names:
		x=names.pop(0)
		if x in seen:
			continue
		y=self.name_to_obj(x)
		if not y:
			raise Utils.WafError("object '%s' was not found in uselib_local (required by '%s')"%(x,self.name))
		if getattr(y,'uselib_local',None):
			lst=y.to_list(y.uselib_local)
			for u in lst:
				if not u in seen:
					names.append(u)
		y.post()
		seen.append(x)
		libname=y.target[y.target.rfind(os.sep)+1:]
		if'cshlib'in y.features or'cprogram'in y.features:
			env.append_value('LIB',libname)
		elif'cstaticlib'in y.features:
			env.append_value('STATICLIB',libname)
		if y.link_task is not None:
			self.link_task.set_run_after(y.link_task)
			dep_nodes=getattr(self.link_task,'dep_nodes',[])
			self.link_task.dep_nodes=dep_nodes+y.link_task.outputs
			tmp_path=y.link_task.outputs[0].parent.bldpath(self.env)
			if not tmp_path in env['LIBPATH']:env.prepend_value('LIBPATH',tmp_path)
		morelibs=y.to_list(y.uselib)
		for v in morelibs:
			if v in uselib:continue
			uselib=[v]+uselib
		if getattr(y,'export_incdirs',None):
			cpppath_st=self.env['CPPPATH_ST']
			for x in self.to_list(y.export_incdirs):
				node=y.path.find_dir(x)
				if not node:
					raise Utils.WafError('object %s: invalid folder %s in export_incdirs'%(y.target,x))
				self.env.append_unique('INC_PATHS',node)
	for x in uselib:
		for v in self.p_flag_vars:
			val=self.env[v+'_'+x]
			if val:self.env.append_value(v,val)
def apply_objdeps(self):
	if not getattr(self,'add_objects',None):return
	seen=[]
	names=self.to_list(self.add_objects)
	while names:
		x=names[0]
		if x in seen:
			names=names[1:]
			continue
		y=self.name_to_obj(x)
		if not y:
			raise Utils.WafError("object '%s' was not found in uselib_local (required by add_objects '%s')"%(x,self.name))
		if getattr(y,'add_objects',None):
			added=0
			lst=y.to_list(y.add_objects)
			lst.reverse()
			for u in lst:
				if u in seen:continue
				added=1
				names=[u]+names
			if added:continue
		y.post()
		seen.append(x)
		for t in y.compiled_tasks:
			self.link_task.inputs.extend(t.outputs)
def apply_obj_vars(self):
	v=self.env
	lib_st=v['LIB_ST']
	staticlib_st=v['STATICLIB_ST']
	libpath_st=v['LIBPATH_ST']
	staticlibpath_st=v['STATICLIBPATH_ST']
	rpath_st=v['RPATH_ST']
	app=v.append_unique
	if v['FULLSTATIC']:
		v.append_value('LINKFLAGS',v['FULLSTATIC_MARKER'])
	for i in v['RPATH']:
		if i and rpath_st:
			app('LINKFLAGS',rpath_st%i)
	for i in v['LIBPATH']:
		app('LINKFLAGS',libpath_st%i)
		app('LINKFLAGS',staticlibpath_st%i)
	if v['STATICLIB']:
		v.append_value('LINKFLAGS',v['STATICLIB_MARKER'])
		k=[(staticlib_st%i)for i in v['STATICLIB']]
		app('LINKFLAGS',k)
	if not v['FULLSTATIC']:
		if v['STATICLIB']or v['LIB']:
			v.append_value('LINKFLAGS',v['SHLIB_MARKER'])
	app('LINKFLAGS',[lib_st%i for i in v['LIB']])
def apply_vnum(self):
	if sys.platform not in('win32','cygwin','darwin'):
		try:
			nums=self.vnum.split('.')
		except AttributeError:
			pass
		else:
			try:name3=self.soname
			except AttributeError:name3=self.link_task.outputs[0].name+'.'+nums[0]
			self.link_task.outputs.append(self.link_task.outputs[0].parent.find_or_declare(name3))
			self.env.append_value('LINKFLAGS',(self.env['SONAME_ST']%name3).split())
def apply_implib(self):
	if win_platform:
		dll=self.link_task.outputs[0]
		implib=dll.parent.find_or_declare(self.env['implib_PATTERN']%os.path.split(self.target)[1])
		self.link_task.outputs.append(implib)
		if sys.platform=='cygwin':
			pass
		elif sys.platform=='win32':
			self.env.append_value('LINKFLAGS',(self.env['IMPLIB_ST']%implib.bldpath(self.env)).split())
		self.link_task.install=install_implib
def process_obj_files(self):
	if not hasattr(self,'obj_files'):return
	for x in self.obj_files:
		node=self.path.find_resource(x)
		self.link_task.inputs.append(node)
def add_obj_file(self,file):
	if not hasattr(self,'obj_files'):self.obj_files=[]
	if not'process_obj_files'in self.meths:self.meths.append('process_obj_files')
	self.obj_files.append(file)
c_attrs={'cxxflag':'CXXFLAGS','cflag':'CCFLAGS','ccflag':'CCFLAGS','linkflag':'LINKFLAGS','ldflag':'LINKFLAGS','lib':'LIB','libpath':'LIBPATH','staticlib':'STATICLIB','staticlibpath':'STATICLIBPATH','rpath':'RPATH','framework':'FRAMEWORK','frameworkpath':'FRAMEWORKPATH'}
def add_extra_flags(self):
	for x in self.__dict__.keys():
		y=x.lower()
		if y[-1]=='s':
			y=y[:-1]
		if c_attrs.get(y,None):
			self.env.append_unique(c_attrs[y],getattr(self,x))
def link_vnum(self):
	clsname=self.__class__.__name__.replace('vnum_','')
	out=self.outputs
	self.outputs=out[1:]
	ret=Task.TaskBase.classes[clsname].__dict__['run'](self)
	self.outputs=out
	if ret:
		return ret
	try:
		os.remove(self.outputs[0].abspath(self.env))
	except OSError:
		pass
	try:
		os.symlink(self.outputs[1].name,self.outputs[0].bldpath(self.env))
	except:
		return 1
def post_dll_link(self):
	if sys.platform=='cygwin':
		try:
			os.remove(self.outputs[1].abspath(self.env))
		except OSError:
			pass
		try:
			os.symlink(self.outputs[0].name,self.outputs[1].bldpath(self.env))
		except:
			return 1

feature('cc','cxx')(default_cc)
before('apply_core')(default_cc)
feature('cprogram','dprogram','cstaticlib','dstaticlib','cshlib','dshlib')(apply_verif)
feature('cprogram','dprogram')(vars_target_cprogram)
before('apply_core')(vars_target_cprogram)
feature('cstaticlib','dstaticlib')(vars_target_cstaticlib)
before('apply_core')(vars_target_cstaticlib)
feature('cshlib','dshlib')(vars_target_cshlib)
before('apply_core')(vars_target_cshlib)
feature('cprogram','dprogram','cstaticlib','dstaticlib','cshlib','dshlib')(install_target_cstaticlib)
after('apply_objdeps','apply_link')(install_target_cstaticlib)
feature('cshlib','dshlib')(install_target_cshlib)
after('apply_link')(install_target_cshlib)
feature('cc','cxx')(apply_incpaths)
after('apply_type_vars','apply_lib_vars','apply_core')(apply_incpaths)
feature('cc','cxx')(apply_type_vars)
after('init_cc','init_cxx')(apply_type_vars)
before('apply_lib_vars')(apply_type_vars)
feature('cprogram','cshlib','cstaticlib')(apply_link)
after('apply_core')(apply_link)
feature('cc','cxx')(apply_lib_vars)
after('apply_link','init_cc','init_cxx')(apply_lib_vars)
feature('cprogram','cstaticlib','cshlib')(apply_objdeps)
after('apply_obj_vars','apply_vnum','apply_implib','apply_link')(apply_objdeps)
feature('cprogram','cshlib','cstaticlib')(apply_obj_vars)
after('apply_lib_vars')(apply_obj_vars)
feature('cshlib')(apply_vnum)
after('apply_link')(apply_vnum)
before('apply_lib_vars')(apply_vnum)
feature('implib')(apply_implib)
after('apply_link')(apply_implib)
before('apply_lib_vars')(apply_implib)
after('apply_link')(process_obj_files)
taskgen(add_obj_file)
feature('cc','cxx')(add_extra_flags)
before('init_cxx','init_cc')(add_extra_flags)
before('apply_lib_vars','apply_obj_vars','apply_incpaths','init_cc')(add_extra_flags)



#!/usr/bin/env python
# vim: set ts=4 sts=4 sw=4 et :
#
# Copyright (c) 2009 Mark Lee <libdesktop-agnostic@lazymalevolence.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.

import sys
import os.path
import tempfile
from desktopagnostic import vfs
import gtk

CONTENT = 'Desktop Agnostic Library'


def main():
    gtk.init_check()
    vfs.init()
    test_launch = 'launch' in sys.argv
    try:
        path = tempfile.gettempdir()
        tmp = vfs.File.for_path(path)
        assert tmp.props.parent is not None
        assert tmp.exists()
        assert tmp.props.file_type == vfs.FILE_TYPE_DIRECTORY
        assert (tmp.props.access_flags & vfs.ACCESS_FLAGS_READ) != 0
        assert tmp.is_readable()
        assert (tmp.props.access_flags & vfs.ACCESS_FLAGS_WRITE) != 0
        assert tmp.is_writable()
        print 'URI: %s' % tmp.props.uri
        print 'Path: %s' % tmp.props.path
        print '# of files: %d' % len(tmp.enumerate_children())
        file_path = os.path.join(path,
                                 '%s-lda-test' % tempfile.gettempprefix())
        tmp_file = vfs.File.for_path(file_path)
        assert tmp_file.props.parent is not None and \
               tmp_file.props.parent.props.uri == tmp.props.uri
        tmp_file.replace_contents(CONTENT)
        assert tmp_file.load_contents() == CONTENT
        if test_launch:
            assert tmp_file.launch()
        file_copy_path = '%s-copy' % file_path
        file_copy = vfs.File.for_path(file_copy_path)
        assert tmp_file.copy(file_copy, True)
        assert CONTENT == file_copy.load_contents()
        if not test_launch:
            assert file_copy.remove()
            assert not file_copy.exists()
            assert tmp_file.remove()
            assert not tmp_file.exists()
    finally:
        vfs.shutdown()

if __name__ == '__main__':
    main()



#! /usr/bin/env python
# encoding: utf-8

import TaskGen
from TaskGen import taskgen,feature
from Constants import*
TaskGen.declare_chain(name='luac',rule='${LUAC} -s -o ${TGT} ${SRC}',ext_in='.lua',ext_out='.luac',reentrant=0,install='LUADIR',)
def init_lua(self):
	self.default_chmod=O755
def detect(conf):
	conf.find_program('luac',var='LUAC',mandatory=True)

feature('lua')(init_lua)



#!/usr/bin/env python

# Copyright (c) 2009 Mark Lee <libdesktop-agnostic@lazymalevolence.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.

import sys
import gtk
from desktopagnostic import fdo
from desktopagnostic import vfs


def main(args):
    vfs.init()
    try:
        if len(args) > 1:
            for arg in args[1:]:
                desktop_file = vfs.File.for_path(arg)
                entry = fdo.DesktopEntry.for_file(desktop_file)
                print 'Entry: %s' % entry.props.name
                print 'Entry exec line: %s', entry.get_string('Exec')
                if entry.exists():
                    pid = entry.launch(0, None)
                    print 'PID: %d' % pid
                else:
                    print 'Entry does not exist!'
        else:
            entry = fdo.DesktopEntry.new()
            entry.props.name = 'hosts file'
            entry.props.entry_type = fdo.DESKTOP_ENTRY_TYPE_LINK
            entry.set_string('URL', 'file:///etc/hosts')
            test_filename = '/tmp/desktop-agnostic-test.desktop'
            desktop_file = vfs.File.for_path(test_filename)
            entry.save(desktop_file)
    finally:
        vfs.shutdown()

if __name__ == '__main__':
    gtk.init_check()
    main(sys.argv)



#! /usr/bin/env python
# encoding: utf-8
import sys
if sys.hexversion < 0x020400f0: from sets import Set as set
import os
import TaskGen,Build,Utils,Task
from Logs import debug
import ccroot
from TaskGen import feature,before,extension,after
g_cc_flag_vars=['CCDEPS','FRAMEWORK','FRAMEWORKPATH','STATICLIB','LIB','LIBPATH','LINKFLAGS','RPATH','CCFLAGS','CPPPATH','CPPFLAGS','CCDEFINES']
EXT_CC=['.c']
g_cc_type_vars=['CCFLAGS','LINKFLAGS']
class cc_taskgen(ccroot.ccroot_abstract):
	pass
def init_cc(self):
	self.p_flag_vars=set(self.p_flag_vars).union(g_cc_flag_vars)
	self.p_type_vars=set(self.p_type_vars).union(g_cc_type_vars)
	if not self.env['CC_NAME']:
		raise Utils.WafError("At least one compiler (gcc, ..) must be selected")
def apply_obj_vars_cc(self):
	env=self.env
	app=env.append_unique
	cpppath_st=env['CPPPATH_ST']
	for i in env['INC_PATHS']:
		app('_CCINCFLAGS',cpppath_st%i.bldpath(env))
		app('_CCINCFLAGS',cpppath_st%i.srcpath(env))
	for i in env['CPPPATH']:
		app('_CCINCFLAGS',cpppath_st%i)
def apply_defines_cc(self):
	self.defines=getattr(self,'defines',[])
	lst=self.to_list(self.defines)+self.to_list(self.env['CCDEFINES'])
	milst=[]
	for defi in lst:
		if not defi in milst:
			milst.append(defi)
	libs=self.to_list(self.uselib)
	for l in libs:
		val=self.env['CCDEFINES_'+l]
		if val:milst+=val
	self.env['DEFLINES']=["%s %s"%(x[0],Utils.trimquotes('='.join(x[1:])))for x in[y.split('=')for y in milst]]
	y=self.env['CCDEFINES_ST']
	self.env['_CCDEFFLAGS']=[y%x for x in milst]
def c_hook(self,node):
	task=self.create_task('cc')
	if getattr(self,'obj_ext',None):
		obj_ext=self.obj_ext
	else:
		obj_ext='_%d.o'%self.idx
	task.inputs=[node]
	task.outputs=[node.change_ext(obj_ext)]
	try:
		self.compiled_tasks.append(task)
	except AttributeError:
		raise Utils.WafError('Have you forgotten to set the feature "cc" on %s?'%str(self))
	return task
cc_str='${CC} ${CCFLAGS} ${CPPFLAGS} ${_CCINCFLAGS} ${_CCDEFFLAGS} ${CC_SRC_F}${SRC} ${CC_TGT_F}${TGT}'
cls=Task.simple_task_type('cc',cc_str,'GREEN',ext_out='.o',ext_in='.c',shell=False)
cls.scan=ccroot.scan
cls.vars.append('CCDEPS')
link_str='${LINK_CC} ${CCLNK_SRC_F}${SRC} ${CCLNK_TGT_F}${TGT} ${LINKFLAGS}'
cls=Task.simple_task_type('cc_link',link_str,color='YELLOW',ext_in='.o',shell=False)
cls.maxjobs=1
cls2=Task.task_type_from_func('vnum_cc_link',ccroot.link_vnum,cls.vars,color='CYAN',ext_in='.o')
cls2.maxjobs=1
link_str='${LINK_CC} ${CCLNK_SRC_F}${SRC} ${CCLNK_TGT_F}${TGT[0].abspath(env)} ${LINKFLAGS}'
cls=Task.simple_task_type('dll_cc_link',link_str,color='YELLOW',ext_in='.o',shell=False)
cls.maxjobs=1
old=cls.run
def run(self):return old(self)or ccroot.post_dll_link(self)
cls.run=run

feature('cc')(init_cc)
before('apply_type_vars')(init_cc)
after('default_cc')(init_cc)
feature('cc')(apply_obj_vars_cc)
after('apply_incpaths')(apply_obj_vars_cc)
feature('cc')(apply_defines_cc)
after('apply_lib_vars')(apply_defines_cc)
extension(EXT_CC)(c_hook)



#! /usr/bin/env python
# encoding: utf-8

import os,sys
import Task
from Configure import conftest
ar_str='${AR} ${ARFLAGS} ${TGT} ${SRC}'
cls=Task.simple_task_type('ar_link_static',ar_str,color='YELLOW',ext_in='.o',shell=False)
cls.maxjobs=1
old=cls.run
def wrap(self):
	try:os.remove(self.outputs[0].abspath(self.env))
	except OSError:pass
	return old(self)
setattr(cls,'run',wrap)
def detect(conf):
	comp=conf.environ.get('AR','')
	if not comp:comp=conf.env['AR']
	if not comp:comp=conf.find_program('ar',var='AR')
	if not comp:return
	ranlib=conf.environ.get('RANLIB','')
	if not ranlib:ranlib=conf.env['RANLIB']
	if not ranlib:ranlib=conf.find_program('ranlib',var='RANLIB')
	if not ranlib:return
	v=conf.env
	v['AR']=comp
	v['ARFLAGS']='rcs'
	v['RANLIB']=ranlib
	v['RANLIBFLAGS']=''
def find_ar(conf):
	v=conf.env
	conf.check_tool('ar')
	if not v['AR']:conf.fatal('ar is required for static libraries - not found')

conftest(find_ar)



#! /usr/bin/env python
# encoding: utf-8

import os,optparse
import Utils,Options,Configure
import ccroot,ar
from Configure import conftest
def find_sxx(conf):
	v=conf.env
	cc=None
	if v['CXX']:cc=v['CXX']
	elif'CXX'in conf.environ:cc=conf.environ['CXX']
	if not cc:cc=conf.find_program('c++',var='CXX')
	if not cc:cc=conf.find_program('CC',var='CXX')
	if not cc:conf.fatal('sunc++ was not found')
	v['CXX']=cc
	v['CXX_NAME']='sun'
def sxx_common_flags(conf):
	v=conf.env
	v['CXX_SRC_F']=''
	v['CXX_TGT_F']=['-c','-o','']
	v['CPPPATH_ST']='-I%s'
	if not v['LINK_CXX']:v['LINK_CXX']=v['CXX']
	v['CXXLNK_SRC_F']=''
	v['CXXLNK_TGT_F']=['-o','']
	v['LIB_ST']='-l%s'
	v['LIBPATH_ST']='-L%s'
	v['STATICLIB_ST']='-l%s'
	v['STATICLIBPATH_ST']='-L%s'
	v['CXXDEFINES_ST']='-D%s'
	v['SONAME_ST']='-Wl,-h -Wl,%s'
	v['SHLIB_MARKER']='-Bdynamic'
	v['STATICLIB_MARKER']='-Bstatic'
	v['program_PATTERN']='%s'
	v['shlib_CXXFLAGS']=['-Kpic','-DPIC']
	v['shlib_LINKFLAGS']=['-G']
	v['shlib_PATTERN']='lib%s.so'
	v['staticlib_LINKFLAGS']=['-Bstatic']
	v['staticlib_PATTERN']='lib%s.a'
detect='''
find_sxx
find_cpp
find_ar
sxx_common_flags
cxx_load_tools
cxx_add_flags
'''

conftest(find_sxx)
conftest(sxx_common_flags)



#! /usr/bin/env python
# encoding: utf-8

import os,sys,imp,types
import Utils,Configure,Options
def detect(conf):
	if getattr(Options.options,'check_dmd_first',None):
		test_for_compiler=['dmd','gdc']
	else:
		test_for_compiler=['gdc','dmd']
	for d_compiler in test_for_compiler:
		conf.check_tool(d_compiler)
		if conf.env['D_COMPILER']:
			conf.check_message("%s"%d_compiler,'',True)
			conf.env["COMPILER_D"]=d_compiler
			return
		conf.check_message("%s"%d_compiler,'',False)
def set_options(opt):
	d_compiler_opts=opt.add_option_group("D Compiler Options")
	d_compiler_opts.add_option('--check-dmd-first',action="store_true",help='checks for the gdc compiler before dmd (default is the other way round)',dest='check_dmd_first',default=False)
	for d_compiler in['gdc','dmd']:
		opt.tool_options('%s'%d_compiler,option_group=d_compiler_opts)




#! /usr/bin/env python
# encoding: utf-8

import TaskGen,Utils,Task
from Logs import error
from TaskGen import before,after,taskgen,feature
flag_vars=['FLAGS','ASSEMBLIES']
def init_cs(self):
	Utils.def_attrs(self,flags='',assemblies='',resources='',uselib='')
def apply_uselib_cs(self):
	if not self.uselib:
		return
	global flag_vars
	for var in self.to_list(self.uselib):
		for v in self.flag_vars:
			val=self.env[v+'_'+var]
			if val:self.env.append_value(v,val)
def apply_cs(self):
	try:self.meths.remove('apply_core')
	except ValueError:pass
	assemblies_flags=[]
	for i in self.to_list(self.assemblies)+self.env['ASSEMBLIES']:
		assemblies_flags+='/r:'+i
	self.env['_ASSEMBLIES']+=assemblies_flags
	for i in self.to_list(self.resources):
		self.env['_RESOURCES'].append('/resource:'+i)
	self.env['_FLAGS']+=self.to_list(self.flags)+self.env['FLAGS']
	curnode=self.path
	nodes=[]
	for i in self.to_list(self.source):
		nodes.append(curnode.find_resource(i))
	task=self.create_task('mcs')
	task.inputs=nodes
	task.set_outputs(self.path.find_or_declare(self.target))
Task.simple_task_type('mcs','${MCS} ${SRC} /out:${TGT} ${_FLAGS} ${_ASSEMBLIES} ${_RESOURCES}',color='YELLOW')
def detect(conf):
	mcs=conf.find_program('mcs',var='MCS')
	if not mcs:mcs=conf.find_program('gmcs',var='MCS')

feature('cs')(init_cs)
feature('cs')(apply_uselib_cs)
after('init_cs')(apply_uselib_cs)
feature('cs')(apply_cs)
after('apply_uselib_cs')(apply_cs)
before('apply_core')(apply_cs)



#! /usr/bin/env python
# encoding: utf-8

import os
all_modifs={}
def modif(dir,name,fun):
	if name=='*':
		lst=os.listdir(dir)+['Tools'+os.sep+x for x in os.listdir(os.path.join(dir,'Tools'))]
		for x in lst:
			if x.endswith('.py'):
				modif(dir,x,fun)
		return
	filename=os.path.join(dir,name)
	f=open(filename,'r')
	txt=f.read()
	f.close()
	txt=fun(txt)
	f=open(filename,'w')
	f.write(txt)
	f.close()
def subst(filename):
	def do_subst(fun):
		global all_modifs
		try:
			all_modifs[filename]+=fun
		except KeyError:
			all_modifs[filename]=[fun]
		return fun
	return do_subst
def r1(code):
	code=code.replace("'iluvcuteoverload'","b'iluvcuteoverload'")
	code=code.replace("ABI=7","ABI=37")
	return code
def r2(code):
	code=code.replace("p.stdin.write('\\n')","p.stdin.write(b'\\n')")
	code=code.replace("out=str(out)","out=out.decode('utf-8')")
	return code
def r3(code):
	code=code.replace("m.update(str(lst))","m.update(str(lst).encode())")
	return code
def r4(code):
	code=code.replace("up(self.__class__.__name__)","up(self.__class__.__name__.encode())")
	code=code.replace("up(self.env.variant())","up(self.env.variant().encode())")
	code=code.replace("up(x.parent.abspath())","up(x.parent.abspath().encode())")
	code=code.replace("up(x.name)","up(x.name.encode())")
	code=code.replace('class TaskBase(object):\n\t__metaclass__=store_task_type','class TaskBase(object, metaclass=store_task_type):')
	code=code.replace('keys=self.cstr_groups.keys()','keys=list(self.cstr_groups.keys())')
	return code
def r5(code):
	code=code.replace("cPickle.dump(data,file,-1)","cPickle.dump(data,file)")
	code=code.replace('for node in src_dir_node.childs.values():','for node in list(src_dir_node.childs.values()):')
	return code
def r6(code):
	code=code.replace('xrange','range')
	code=code.replace('iteritems','items')
	code=code.replace('maxint','maxsize')
	code=code.replace('iterkeys','keys')
	code=code.replace('Error,e:','Error as e:')
	code=code.replace('Exception,e:','Exception as e:')
	return code
def r7(code):
	code=code.replace('class task_gen(object):\n\t__metaclass__=register_obj','class task_gen(object, metaclass=register_obj):')
	return code
def fixdir(dir):
	global all_modifs
	for k in all_modifs:
		for v in all_modifs[k]:
			modif(os.path.join(dir,'wafadmin'),k,v)

subst('Constants.py')(r1)
subst('Tools/ccroot.py')(r2)
subst('Utils.py')(r3)
subst('Task.py')(r4)
subst('Build.py')(r5)
subst('*')(r6)
subst('TaskGen.py')(r7)



#!/usr/bin/env python
#
# Copyright (c) 2009 Mark Lee <libdesktop-agnostic@lazymalevolence.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.

import gtk
import desktopagnostic
from desktopagnostic.ui import ColorButton


class TestColorButton(gtk.Window):

    def __init__(self):
        super(TestColorButton, self).__init__()
        self.connect('delete-event', gtk.main_quit)
        color = desktopagnostic.Color.from_string('green')
        color.props.alpha = 2 ** 15 - 1
        button = ColorButton.with_color(color)
        button.connect('color-set', self.on_color_set)
        self.add(button)
        self.show_all()

    def on_color_set(self, button):
        print str(button.props.da_color)
        gtk.main_quit()

if __name__ == '__main__':
    tcb = TestColorButton()
    gtk.main()

# vim:ts=4:sts=4:sw=4:et:ai:cindent:



#!/usr/bin/env python
#
# Copyright(c) 2009 Mark Lee <libdesktop-agnostic@lazymalevolence.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.

import glib
import time
import unittest
from desktopagnostic import config

'''
Map format: type name => (group, key, default, set value)
'''
type_key_map = {
    'bool': ('numeric', 'boolean', True, False),
    'int': ('numeric', 'integer', 3, 10),
    'float': ('numeric', 'float', 3.14, 2.718),
    'string': ('misc', 'string', 'Foo bar', 'Quux baz'),
}

'''
Map format: type name => (default, set value)
'''
list_value_map = {
    'bool': ([True, False], [False, True, False]),
    'int': ([1, 2, 3], [10, 20, 30]),
    'float': ([1.618, 2.718, 3.141], [10.5, 20.6, 30.7]),
    'string': (['foo', 'bar'], ['Quux', 'Baz', 'Foo']),
}


def create_type_tests(type_name, group, key, default, set_val):
    list_default, list_set_val = list_value_map[type_name]

    def test_default(self):
        self.check_values(group, key, default, type_name)

    def test_default_list(self):
        self.check_values('list', key, list_default, 'list')

    def test_set(self):
        self.check_set_values(group, key, set_val, type_name, False)

    def test_set_list(self):
        self.check_set_values('list', key, list_set_val, 'list', False)

    def test_set_value(self):
        self.check_set_values(group, key, set_val, type_name, True)

    def test_set_list_value(self):
        self.check_set_values('list', key, list_set_val, 'list', True)
    return {
        'test_default_%s' % type_name: test_default,
        'test_default_list_%s' % type_name: test_default_list,
        'test_set_%s' % type_name: test_set,
        'test_set_list_%s' % type_name: test_set_list,
        'test_set_value_%s' % type_name: test_set_value,
        'test_set_list_value_%s' % type_name: test_set_list_value}


class TestConfigClient(unittest.TestCase):

    def setUp(self):
        self.ml = glib.MainLoop()
        self.notify_counter = 0
        self.client = config.Client('../test-config.schema-ini')
        self.client.reset(False)

    def check_values(self, group, key, expected, type_name):
        get_func = getattr(self.client, 'get_%s' % type_name)
        actual = get_func(group, key)
        actual_value = self.client.get_value(group, key)
        if isinstance(expected, float):
            expected = round(expected, 3)
            actual = round(actual, 3)
            actual_value = round(actual_value, 3)
        elif isinstance(expected, list) and isinstance(expected[0], float):
            expected = [round(x, 3) for x in expected]
            actual = [round(x, 3) for x in actual]
            actual_value = [round(x, 3) for x in actual_value]
        ERROR_MSG = 'Fail! Expected value: %s, actual: %s'
        self.assertEqual(expected, actual,
                         ERROR_MSG % (str(expected), str(actual)))
        self.assertEqual(actual, actual_value)

    def check_set_values(self, group, key, expected, type_name, set_value):
        if set_value:
            set_func = self.client.set_value
        else:
            set_func = getattr(self.client, 'set_%s' % type_name)
        set_func(group, key, expected)
        self.check_values(group, key, expected, type_name)

    def string_changed(self, group, key, value):
        self.notify_counter += 1

    def string_changed2(self, group, key, value):
        if 'quux' in value:
            self.notify_counter += 3

    def check_notify(self, ctx, string, expected_counter):
        self.client.set_string('misc', 'string', string)
        # wait for the notification
        time.sleep(0.25)
        while ctx.pending():
            ctx.iteration()
        self.assertEqual(self.notify_counter, expected_counter)

    def test_notify(self):
        ctx = self.ml.get_context()

        self.client.notify_add('misc', 'string', self.string_changed)
        self.client.notify_add('misc', 'string', self.string_changed2)

        self.check_notify(ctx, 'Bar foo', 1)

        self.check_notify(ctx, 'Foo quux', 5)

        self.client.notify_remove('misc', 'string', self.string_changed)
        self.check_notify(ctx, 'Bar quux', 8)

        self.client.notify_remove('misc', 'string', self.string_changed2)
        self.check_notify(ctx, 'Baz foo', 8)

# add the type-specific tests to the testcase
for type_name, data in type_key_map.iteritems():
    methods = create_type_tests(type_name, *data)
    for name, method in methods.iteritems():
        setattr(TestConfigClient, name, method)

if __name__ == '__main__':
    unittest.main()



#! /usr/bin/env python
# encoding: utf-8
import sys
if sys.hexversion < 0x020400f0: from sets import Set as set
import TaskGen,Task,Utils
from Logs import debug
import ccroot
from TaskGen import feature,before,extension,after
g_cxx_flag_vars=['CXXDEPS','FRAMEWORK','FRAMEWORKPATH','STATICLIB','LIB','LIBPATH','LINKFLAGS','RPATH','CXXFLAGS','CCFLAGS','CPPPATH','CPPFLAGS','CXXDEFINES']
EXT_CXX=['.cpp','.cc','.cxx','.C','.c++']
g_cxx_type_vars=['CXXFLAGS','LINKFLAGS']
class cxx_taskgen(ccroot.ccroot_abstract):
	pass
def init_cxx(self):
	if not'cc'in self.features:
		self.mappings['.c']=TaskGen.task_gen.mappings['.cxx']
	self.p_flag_vars=set(self.p_flag_vars).union(g_cxx_flag_vars)
	self.p_type_vars=set(self.p_type_vars).union(g_cxx_type_vars)
	if not self.env['CXX_NAME']:
		raise Utils.WafError("At least one compiler (g++, ..) must be selected")
def apply_obj_vars_cxx(self):
	env=self.env
	app=env.append_unique
	cxxpath_st=env['CPPPATH_ST']
	for i in env['INC_PATHS']:
		app('_CXXINCFLAGS',cxxpath_st%i.bldpath(env))
		app('_CXXINCFLAGS',cxxpath_st%i.srcpath(env))
	for i in env['CPPPATH']:
		app('_CXXINCFLAGS',cxxpath_st%i)
def apply_defines_cxx(self):
	self.defines=getattr(self,'defines',[])
	lst=self.to_list(self.defines)+self.to_list(self.env['CXXDEFINES'])
	milst=[]
	for defi in lst:
		if not defi in milst:
			milst.append(defi)
	libs=self.to_list(self.uselib)
	for l in libs:
		val=self.env['CXXDEFINES_'+l]
		if val:milst+=self.to_list(val)
	self.env['DEFLINES']=["%s %s"%(x[0],Utils.trimquotes('='.join(x[1:])))for x in[y.split('=')for y in milst]]
	y=self.env['CXXDEFINES_ST']
	self.env['_CXXDEFFLAGS']=[y%x for x in milst]
def cxx_hook(self,node):
	task=self.create_task('cxx')
	if getattr(self,'obj_ext',None):
		obj_ext=self.obj_ext
	else:
		obj_ext='_%d.o'%self.idx
	task.inputs=[node]
	task.outputs=[node.change_ext(obj_ext)]
	try:
		self.compiled_tasks.append(task)
	except AttributeError:
		raise Utils.WafError('Have you forgotten to set the feature "cxx" on %s?'%str(self))
	return task
cxx_str='${CXX} ${CXXFLAGS} ${CPPFLAGS} ${_CXXINCFLAGS} ${_CXXDEFFLAGS} ${CXX_SRC_F}${SRC} ${CXX_TGT_F}${TGT}'
cls=Task.simple_task_type('cxx',cxx_str,color='GREEN',ext_out='.o',ext_in='.cxx',shell=False)
cls.scan=ccroot.scan
cls.vars.append('CXXDEPS')
link_str='${LINK_CXX} ${CXXLNK_SRC_F}${SRC} ${CXXLNK_TGT_F}${TGT} ${LINKFLAGS}'
cls=Task.simple_task_type('cxx_link',link_str,color='YELLOW',ext_in='.o',shell=False)
cls.maxjobs=1
cls2=Task.task_type_from_func('vnum_cxx_link',ccroot.link_vnum,cls.vars,color='CYAN',ext_in='.o')
cls2.maxjobs=1
link_str='${LINK_CXX} ${CXXLNK_SRC_F}${SRC} ${CXXLNK_TGT_F}${TGT[0].abspath(env)} ${LINKFLAGS}'
cls=Task.simple_task_type('dll_cxx_link',link_str,color='YELLOW',ext_in='.o',shell=False)
cls.maxjobs=1
old=cls.run
def run(self):return old(self)or ccroot.post_dll_link(self)
cls.run=run

feature('cxx')(init_cxx)
before('apply_type_vars')(init_cxx)
after('default_cc')(init_cxx)
feature('cxx')(apply_obj_vars_cxx)
after('apply_incpaths')(apply_obj_vars_cxx)
feature('cxx')(apply_defines_cxx)
after('apply_lib_vars')(apply_defines_cxx)
extension(EXT_CXX)(cxx_hook)



#! /usr/bin/env python
# encoding: utf-8

import TaskGen
def decide_ext(self,node):
	if'cxx'in self.features:return'.lex.cc'
	else:return'.lex.c'
TaskGen.declare_chain(name='flex',rule='${FLEX} -o${TGT} ${FLEXFLAGS} ${SRC}',ext_in='.l',decider=decide_ext,before='cc cxx',)
def detect(conf):
	conf.find_program('flex',var='FLEX',mandatory=True)
	v=conf.env
	v['FLEXFLAGS']=''




#!/usr/bin/env python
#
# Copyright (c) 2009 Mark Lee <libdesktop-agnostic@lazymalevolence.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.

import gtk

# needed to have a readable color string
import desktopagnostic
from desktopagnostic.ui import ColorButton


def on_color_set(button):
    print str(button.props.da_color)
    gtk.main_quit()

if __name__ == '__main__':
    builder = gtk.Builder()
    builder.add_from_file('../test-ui-color-button-gtkbuilder.ui')
    window = builder.get_object('window1')
    window.connect('delete-event', gtk.main_quit)
    button = window.get_child()
    button.connect('color-set', on_color_set)
    window.show_all()
    gtk.main()

# vim:ts=4:sts=4:sw=4:et:ai:cindent:



#! /usr/bin/env python
# encoding: utf-8

import sys
mswindows=(sys.platform=="win32")
import os
import types
import traceback
import gc
class CalledProcessError(Exception):
	def __init__(self,returncode,cmd):
		self.returncode=returncode
		self.cmd=cmd
	def __str__(self):
		return"Command '%s' returned non-zero exit status %d"%(self.cmd,self.returncode)
if mswindows:
	import threading
	import msvcrt
	if 0:
		import pywintypes
		from win32api import GetStdHandle,STD_INPUT_HANDLE,STD_OUTPUT_HANDLE,STD_ERROR_HANDLE
		from win32api import GetCurrentProcess,DuplicateHandle,GetModuleFileName,GetVersion
		from win32con import DUPLICATE_SAME_ACCESS,SW_HIDE
		from win32pipe import CreatePipe
		from win32process import CreateProcess,STARTUPINFO,GetExitCodeProcess,STARTF_USESTDHANDLES,STARTF_USESHOWWINDOW,CREATE_NEW_CONSOLE
		from win32event import WaitForSingleObject,INFINITE,WAIT_OBJECT_0
	else:
		from _subprocess import*
		class STARTUPINFO:
			dwFlags=0
			hStdInput=None
			hStdOutput=None
			hStdError=None
			wShowWindow=0
		class pywintypes:
			error=IOError
else:
	import select
	import errno
	import fcntl
	import pickle
__all__=["Popen","PIPE","STDOUT","call","check_call","CalledProcessError"]
try:
	MAXFD=os.sysconf("SC_OPEN_MAX")
except:
	MAXFD=256
try:
	False
except NameError:
	False=0
	True=1
_active=[]
def _cleanup():
	for inst in _active[:]:
		if inst.poll(_deadstate=sys.maxint)>=0:
			try:
				_active.remove(inst)
			except ValueError:
				pass
PIPE=-1
STDOUT=-2
def call(*popenargs,**kwargs):
	return Popen(*popenargs,**kwargs).wait()
def check_call(*popenargs,**kwargs):
	retcode=call(*popenargs,**kwargs)
	cmd=kwargs.get("args")
	if cmd is None:
		cmd=popenargs[0]
	if retcode:
		raise CalledProcessError(retcode,cmd)
	return retcode
def list2cmdline(seq):
	result=[]
	needquote=False
	for arg in seq:
		bs_buf=[]
		if result:
			result.append(' ')
		needquote=(" "in arg)or("\t"in arg)or arg==""
		if needquote:
			result.append('"')
		for c in arg:
			if c=='\\':
				bs_buf.append(c)
			elif c=='"':
				result.append('\\'*len(bs_buf)*2)
				bs_buf=[]
				result.append('\\"')
			else:
				if bs_buf:
					result.extend(bs_buf)
					bs_buf=[]
				result.append(c)
		if bs_buf:
			result.extend(bs_buf)
		if needquote:
			result.extend(bs_buf)
			result.append('"')
	return''.join(result)
class Popen(object):
	def __init__(self,args,bufsize=0,executable=None,stdin=None,stdout=None,stderr=None,preexec_fn=None,close_fds=False,shell=False,cwd=None,env=None,universal_newlines=False,startupinfo=None,creationflags=0):
		_cleanup()
		self._child_created=False
		if not isinstance(bufsize,(int,long)):
			raise TypeError("bufsize must be an integer")
		if mswindows:
			if preexec_fn is not None:
				raise ValueError("preexec_fn is not supported on Windows platforms")
			if close_fds:
				raise ValueError("close_fds is not supported on Windows platforms")
		else:
			if startupinfo is not None:
				raise ValueError("startupinfo is only supported on Windows platforms")
			if creationflags!=0:
				raise ValueError("creationflags is only supported on Windows platforms")
		self.stdin=None
		self.stdout=None
		self.stderr=None
		self.pid=None
		self.returncode=None
		self.universal_newlines=universal_newlines
		(p2cread,p2cwrite,c2pread,c2pwrite,errread,errwrite)=self._get_handles(stdin,stdout,stderr)
		self._execute_child(args,executable,preexec_fn,close_fds,cwd,env,universal_newlines,startupinfo,creationflags,shell,p2cread,p2cwrite,c2pread,c2pwrite,errread,errwrite)
		if mswindows:
			if stdin is None and p2cwrite is not None:
				os.close(p2cwrite)
				p2cwrite=None
			if stdout is None and c2pread is not None:
				os.close(c2pread)
				c2pread=None
			if stderr is None and errread is not None:
				os.close(errread)
				errread=None
		if p2cwrite:
			self.stdin=os.fdopen(p2cwrite,'wb',bufsize)
		if c2pread:
			if universal_newlines:
				self.stdout=os.fdopen(c2pread,'rU',bufsize)
			else:
				self.stdout=os.fdopen(c2pread,'rb',bufsize)
		if errread:
			if universal_newlines:
				self.stderr=os.fdopen(errread,'rU',bufsize)
			else:
				self.stderr=os.fdopen(errread,'rb',bufsize)
	def _translate_newlines(self,data):
		data=data.replace("\r\n","\n")
		data=data.replace("\r","\n")
		return data
	def __del__(self,sys=sys):
		if not self._child_created:
			return
		self.poll(_deadstate=sys.maxint)
		if self.returncode is None and _active is not None:
			_active.append(self)
	def communicate(self,input=None):
		if[self.stdin,self.stdout,self.stderr].count(None)>=2:
			stdout=None
			stderr=None
			if self.stdin:
				if input:
					self.stdin.write(input)
				self.stdin.close()
			elif self.stdout:
				stdout=self.stdout.read()
			elif self.stderr:
				stderr=self.stderr.read()
			self.wait()
			return(stdout,stderr)
		return self._communicate(input)
	if mswindows:
		def _get_handles(self,stdin,stdout,stderr):
			if stdin is None and stdout is None and stderr is None:
				return(None,None,None,None,None,None)
			p2cread,p2cwrite=None,None
			c2pread,c2pwrite=None,None
			errread,errwrite=None,None
			if stdin is None:
				p2cread=GetStdHandle(STD_INPUT_HANDLE)
			if p2cread is not None:
				pass
			elif stdin is None or stdin==PIPE:
				p2cread,p2cwrite=CreatePipe(None,0)
				p2cwrite=p2cwrite.Detach()
				p2cwrite=msvcrt.open_osfhandle(p2cwrite,0)
			elif isinstance(stdin,int):
				p2cread=msvcrt.get_osfhandle(stdin)
			else:
				p2cread=msvcrt.get_osfhandle(stdin.fileno())
			p2cread=self._make_inheritable(p2cread)
			if stdout is None:
				c2pwrite=GetStdHandle(STD_OUTPUT_HANDLE)
			if c2pwrite is not None:
				pass
			elif stdout is None or stdout==PIPE:
				c2pread,c2pwrite=CreatePipe(None,0)
				c2pread=c2pread.Detach()
				c2pread=msvcrt.open_osfhandle(c2pread,0)
			elif isinstance(stdout,int):
				c2pwrite=msvcrt.get_osfhandle(stdout)
			else:
				c2pwrite=msvcrt.get_osfhandle(stdout.fileno())
			c2pwrite=self._make_inheritable(c2pwrite)
			if stderr is None:
				errwrite=GetStdHandle(STD_ERROR_HANDLE)
			if errwrite is not None:
				pass
			elif stderr is None or stderr==PIPE:
				errread,errwrite=CreatePipe(None,0)
				errread=errread.Detach()
				errread=msvcrt.open_osfhandle(errread,0)
			elif stderr==STDOUT:
				errwrite=c2pwrite
			elif isinstance(stderr,int):
				errwrite=msvcrt.get_osfhandle(stderr)
			else:
				errwrite=msvcrt.get_osfhandle(stderr.fileno())
			errwrite=self._make_inheritable(errwrite)
			return(p2cread,p2cwrite,c2pread,c2pwrite,errread,errwrite)
		def _make_inheritable(self,handle):
			return DuplicateHandle(GetCurrentProcess(),handle,GetCurrentProcess(),0,1,DUPLICATE_SAME_ACCESS)
		def _find_w9xpopen(self):
			w9xpopen=os.path.join(os.path.dirname(GetModuleFileName(0)),"w9xpopen.exe")
			if not os.path.exists(w9xpopen):
				w9xpopen=os.path.join(os.path.dirname(sys.exec_prefix),"w9xpopen.exe")
				if not os.path.exists(w9xpopen):
					raise RuntimeError("Cannot locate w9xpopen.exe, which is needed for Popen to work with your shell or platform.")
			return w9xpopen
		def _execute_child(self,args,executable,preexec_fn,close_fds,cwd,env,universal_newlines,startupinfo,creationflags,shell,p2cread,p2cwrite,c2pread,c2pwrite,errread,errwrite):
			if not isinstance(args,types.StringTypes):
				args=list2cmdline(args)
			if startupinfo is None:
				startupinfo=STARTUPINFO()
			if None not in(p2cread,c2pwrite,errwrite):
				startupinfo.dwFlags|=STARTF_USESTDHANDLES
				startupinfo.hStdInput=p2cread
				startupinfo.hStdOutput=c2pwrite
				startupinfo.hStdError=errwrite
			if shell:
				startupinfo.dwFlags|=STARTF_USESHOWWINDOW
				startupinfo.wShowWindow=SW_HIDE
				comspec=os.environ.get("COMSPEC","cmd.exe")
				args=comspec+" /c "+args
				if(GetVersion()>=0x80000000L or os.path.basename(comspec).lower()=="command.com"):
					w9xpopen=self._find_w9xpopen()
					args='"%s" %s'%(w9xpopen,args)
					creationflags|=CREATE_NEW_CONSOLE
			try:
				hp,ht,pid,tid=CreateProcess(executable,args,None,None,1,creationflags,env,cwd,startupinfo)
			except pywintypes.error,e:
				raise WindowsError(*e.args)
			self._child_created=True
			self._handle=hp
			self.pid=pid
			ht.Close()
			if p2cread is not None:
				p2cread.Close()
			if c2pwrite is not None:
				c2pwrite.Close()
			if errwrite is not None:
				errwrite.Close()
		def poll(self,_deadstate=None):
			if self.returncode is None:
				if WaitForSingleObject(self._handle,0)==WAIT_OBJECT_0:
					self.returncode=GetExitCodeProcess(self._handle)
			return self.returncode
		def wait(self):
			if self.returncode is None:
				obj=WaitForSingleObject(self._handle,INFINITE)
				self.returncode=GetExitCodeProcess(self._handle)
			return self.returncode
		def _readerthread(self,fh,buffer):
			buffer.append(fh.read())
		def _communicate(self,input):
			stdout=None
			stderr=None
			if self.stdout:
				stdout=[]
				stdout_thread=threading.Thread(target=self._readerthread,args=(self.stdout,stdout))
				stdout_thread.setDaemon(True)
				stdout_thread.start()
			if self.stderr:
				stderr=[]
				stderr_thread=threading.Thread(target=self._readerthread,args=(self.stderr,stderr))
				stderr_thread.setDaemon(True)
				stderr_thread.start()
			if self.stdin:
				if input is not None:
					self.stdin.write(input)
				self.stdin.close()
			if self.stdout:
				stdout_thread.join()
			if self.stderr:
				stderr_thread.join()
			if stdout is not None:
				stdout=stdout[0]
			if stderr is not None:
				stderr=stderr[0]
			if self.universal_newlines and hasattr(file,'newlines'):
				if stdout:
					stdout=self._translate_newlines(stdout)
				if stderr:
					stderr=self._translate_newlines(stderr)
			self.wait()
			return(stdout,stderr)
	else:
		def _get_handles(self,stdin,stdout,stderr):
			p2cread,p2cwrite=None,None
			c2pread,c2pwrite=None,None
			errread,errwrite=None,None
			if stdin is None:
				pass
			elif stdin==PIPE:
				p2cread,p2cwrite=os.pipe()
			elif isinstance(stdin,int):
				p2cread=stdin
			else:
				p2cread=stdin.fileno()
			if stdout is None:
				pass
			elif stdout==PIPE:
				c2pread,c2pwrite=os.pipe()
			elif isinstance(stdout,int):
				c2pwrite=stdout
			else:
				c2pwrite=stdout.fileno()
			if stderr is None:
				pass
			elif stderr==PIPE:
				errread,errwrite=os.pipe()
			elif stderr==STDOUT:
				errwrite=c2pwrite
			elif isinstance(stderr,int):
				errwrite=stderr
			else:
				errwrite=stderr.fileno()
			return(p2cread,p2cwrite,c2pread,c2pwrite,errread,errwrite)
		def _set_cloexec_flag(self,fd):
			try:
				cloexec_flag=fcntl.FD_CLOEXEC
			except AttributeError:
				cloexec_flag=1
			old=fcntl.fcntl(fd,fcntl.F_GETFD)
			fcntl.fcntl(fd,fcntl.F_SETFD,old|cloexec_flag)
		def _close_fds(self,but):
			for i in xrange(3,MAXFD):
				if i==but:
					continue
				try:
					os.close(i)
				except:
					pass
		def _execute_child(self,args,executable,preexec_fn,close_fds,cwd,env,universal_newlines,startupinfo,creationflags,shell,p2cread,p2cwrite,c2pread,c2pwrite,errread,errwrite):
			if isinstance(args,types.StringTypes):
				args=[args]
			else:
				args=list(args)
			if shell:
				args=["/bin/sh","-c"]+args
			if executable is None:
				executable=args[0]
			errpipe_read,errpipe_write=os.pipe()
			self._set_cloexec_flag(errpipe_write)
			gc_was_enabled=gc.isenabled()
			gc.disable()
			try:
				self.pid=os.fork()
			except:
				if gc_was_enabled:
					gc.enable()
				raise
			self._child_created=True
			if self.pid==0:
				try:
					if p2cwrite:
						os.close(p2cwrite)
					if c2pread:
						os.close(c2pread)
					if errread:
						os.close(errread)
					os.close(errpipe_read)
					if p2cread:
						os.dup2(p2cread,0)
					if c2pwrite:
						os.dup2(c2pwrite,1)
					if errwrite:
						os.dup2(errwrite,2)
					if p2cread and p2cread not in(0,):
						os.close(p2cread)
					if c2pwrite and c2pwrite not in(p2cread,1):
						os.close(c2pwrite)
					if errwrite and errwrite not in(p2cread,c2pwrite,2):
						os.close(errwrite)
					if close_fds:
						self._close_fds(but=errpipe_write)
					if cwd is not None:
						os.chdir(cwd)
					if preexec_fn:
						apply(preexec_fn)
					if env is None:
						os.execvp(executable,args)
					else:
						os.execvpe(executable,args,env)
				except:
					exc_type,exc_value,tb=sys.exc_info()
					exc_lines=traceback.format_exception(exc_type,exc_value,tb)
					exc_value.child_traceback=''.join(exc_lines)
					os.write(errpipe_write,pickle.dumps(exc_value))
				os._exit(255)
			if gc_was_enabled:
				gc.enable()
			os.close(errpipe_write)
			if p2cread and p2cwrite:
				os.close(p2cread)
			if c2pwrite and c2pread:
				os.close(c2pwrite)
			if errwrite and errread:
				os.close(errwrite)
			data=os.read(errpipe_read,1048576)
			os.close(errpipe_read)
			if data!="":
				os.waitpid(self.pid,0)
				child_exception=pickle.loads(data)
				raise child_exception
		def _handle_exitstatus(self,sts):
			if os.WIFSIGNALED(sts):
				self.returncode=-os.WTERMSIG(sts)
			elif os.WIFEXITED(sts):
				self.returncode=os.WEXITSTATUS(sts)
			else:
				raise RuntimeError("Unknown child exit status!")
		def poll(self,_deadstate=None):
			if self.returncode is None:
				try:
					pid,sts=os.waitpid(self.pid,os.WNOHANG)
					if pid==self.pid:
						self._handle_exitstatus(sts)
				except os.error:
					if _deadstate is not None:
						self.returncode=_deadstate
			return self.returncode
		def wait(self):
			if self.returncode is None:
				pid,sts=os.waitpid(self.pid,0)
				self._handle_exitstatus(sts)
			return self.returncode
		def _communicate(self,input):
			read_set=[]
			write_set=[]
			stdout=None
			stderr=None
			if self.stdin:
				self.stdin.flush()
				if input:
					write_set.append(self.stdin)
				else:
					self.stdin.close()
			if self.stdout:
				read_set.append(self.stdout)
				stdout=[]
			if self.stderr:
				read_set.append(self.stderr)
				stderr=[]
			input_offset=0
			while read_set or write_set:
				rlist,wlist,xlist=select.select(read_set,write_set,[])
				if self.stdin in wlist:
					bytes_written=os.write(self.stdin.fileno(),buffer(input,input_offset,512))
					input_offset+=bytes_written
					if input_offset>=len(input):
						self.stdin.close()
						write_set.remove(self.stdin)
				if self.stdout in rlist:
					data=os.read(self.stdout.fileno(),1024)
					if data=="":
						self.stdout.close()
						read_set.remove(self.stdout)
					stdout.append(data)
				if self.stderr in rlist:
					data=os.read(self.stderr.fileno(),1024)
					if data=="":
						self.stderr.close()
						read_set.remove(self.stderr)
					stderr.append(data)
			if stdout is not None:
				stdout=''.join(stdout)
			if stderr is not None:
				stderr=''.join(stderr)
			if self.universal_newlines and hasattr(file,'newlines'):
				if stdout:
					stdout=self._translate_newlines(stdout)
				if stderr:
					stderr=self._translate_newlines(stderr)
			self.wait()
			return(stdout,stderr)




#! /usr/bin/env python
# encoding: utf-8

import os,optparse
import Utils,Options,Configure
import ccroot,ar
from Configure import conftest
def find_scc(conf):
	v=conf.env
	cc=None
	if v['CC']:cc=v['CC']
	elif'CC'in conf.environ:cc=conf.environ['CC']
	if not cc:cc=conf.find_program('cc',var='CC')
	if not cc:conf.fatal('suncc was not found')
	try:
		if not Utils.cmd_output('%s -flags'%cc):
			conf.fatal('suncc %r was not found'%cc)
	except ValueError:
		conf.fatal('suncc -flags could not be executed')
	v['CC']=cc
	v['CC_NAME']='sun'
def scc_common_flags(conf):
	v=conf.env
	v['CC_SRC_F']=''
	v['CC_TGT_F']=['-c','-o','']
	v['CPPPATH_ST']='-I%s'
	if not v['LINK_CC']:v['LINK_CC']=v['CC']
	v['CCLNK_SRC_F']=''
	v['CCLNK_TGT_F']=['-o','']
	v['LIB_ST']='-l%s'
	v['LIBPATH_ST']='-L%s'
	v['STATICLIB_ST']='-l%s'
	v['STATICLIBPATH_ST']='-L%s'
	v['CCDEFINES_ST']='-D%s'
	v['SONAME_ST']='-Wl,-h -Wl,%s'
	v['SHLIB_MARKER']='-Bdynamic'
	v['STATICLIB_MARKER']='-Bstatic'
	v['program_PATTERN']='%s'
	v['shlib_CCFLAGS']=['-Kpic','-DPIC']
	v['shlib_LINKFLAGS']=['-G']
	v['shlib_PATTERN']='lib%s.so'
	v['staticlib_LINKFLAGS']=['-Bstatic']
	v['staticlib_PATTERN']='lib%s.a'
detect='''
find_scc
find_cpp
find_ar
scc_common_flags
cc_load_tools
cc_add_flags
'''

conftest(find_scc)
conftest(scc_common_flags)



#! /usr/bin/env python
# encoding: utf-8

import os,re
import Utils,TaskGen,Task,Runner,Build
from TaskGen import feature,before
from Logs import error,warn,debug
re_tex=re.compile(r'\\(?P<type>include|input|import|bringin){(?P<file>[^{}]*)}',re.M)
def scan(self):
	node=self.inputs[0]
	env=self.env
	nodes=[]
	names=[]
	if not node:return(nodes,names)
	code=Utils.readf(node.abspath(env))
	curdirnode=self.curdirnode
	abs=curdirnode.abspath()
	for match in re_tex.finditer(code):
		path=match.group('file')
		if path:
			for k in['','.tex','.ltx']:
				debug('tex: trying %s%s'%(path,k))
				try:
					os.stat(abs+os.sep+path+k)
				except OSError:
					continue
				found=path+k
				node=curdirnode.find_resource(found)
				if node:
					nodes.append(node)
			else:
				debug('tex: could not find %s'%path)
				names.append(path)
	debug("tex: found the following : %s and names %s"%(nodes,names))
	return(nodes,names)
g_bibtex_re=re.compile('bibdata',re.M)
def tex_build(task,command='LATEX'):
	env=task.env
	bld=task.generator.bld
	com='%s %s'%(env[command],env.get_flat(command+'FLAGS'))
	if not env['PROMPT_LATEX']:com="%s %s"%(com,'-interaction=batchmode')
	node=task.inputs[0]
	reldir=node.bld_dir(env)
	srcfile=node.srcpath(env)
	lst=[]
	for c in Utils.split_path(reldir):
		if c:lst.append('..')
	sr=os.path.join(*(lst+[srcfile]))
	sr2=os.path.join(*(lst+[node.parent.srcpath(env)]))
	aux_node=node.change_ext('.aux')
	idx_node=node.change_ext('.idx')
	hash=''
	old_hash=''
	nm=aux_node.name
	docuname=nm[:len(nm)-4]
	latex_compile_cmd='cd %s && TEXINPUTS=%s:$TEXINPUTS %s %s'%(reldir,sr2,com,sr)
	warn('first pass on %s'%command)
	ret=bld.exec_command(latex_compile_cmd)
	if ret:return ret
	try:
		ct=Utils.readf(aux_node.abspath(env))
	except(OSError,IOError):
		error('error bibtex scan')
	else:
		fo=g_bibtex_re.findall(ct)
		if fo:
			bibtex_compile_cmd='cd %s && BIBINPUTS=%s:$BIBINPUTS %s %s'%(reldir,sr2,env['BIBTEX'],docuname)
			warn('calling bibtex')
			ret=bld.exec_command(bibtex_compile_cmd)
			if ret:
				error('error when calling bibtex %s'%bibtex_compile_cmd)
				return ret
	try:
		idx_path=idx_node.abspath(env)
		os.stat(idx_path)
	except OSError:
		error('error file.idx scan')
	else:
		makeindex_compile_cmd='cd %s && %s %s'%(reldir,env['MAKEINDEX'],idx_path)
		warn('calling makeindex')
		ret=bld.exec_command(makeindex_compile_cmd)
		if ret:
			error('error when calling makeindex %s'%makeindex_compile_cmd)
			return ret
	i=0
	while i<10:
		i+=1
		old_hash=hash
		try:
			hash=Utils.h_file(aux_node.abspath(env))
		except KeyError:
			error('could not read aux.h -> %s'%aux_node.abspath(env))
			pass
		if hash and hash==old_hash:break
		warn('calling %s'%command)
		ret=bld.exec_command(latex_compile_cmd)
		if ret:
			error('error when calling %s %s'%(command,latex_compile_cmd))
			return ret
	return 0
latex_vardeps=['LATEX','LATEXFLAGS']
def latex_build(task):
	return tex_build(task,'LATEX')
pdflatex_vardeps=['PDFLATEX','PDFLATEXFLAGS']
def pdflatex_build(task):
	return tex_build(task,'PDFLATEX')
class tex_taskgen(TaskGen.task_gen):
	def __init__(self,*k,**kw):
		TaskGen.task_gen.__init__(self,*k,**kw)
def apply_tex(self):
	if not getattr(self,'type',None)in['latex','pdflatex']:
		self.type='pdflatex'
	tree=self.bld
	outs=Utils.to_list(getattr(self,'outs',[]))
	self.env['PROMPT_LATEX']=getattr(self,'prompt',1)
	deps_lst=[]
	if getattr(self,'deps',None):
		deps=self.to_list(self.deps)
		for filename in deps:
			n=self.path.find_resource(filename)
			if not n in deps_lst:deps_lst.append(n)
	self.source=self.to_list(self.source)
	for filename in self.source:
		base,ext=os.path.splitext(filename)
		node=self.path.find_resource(filename)
		if not node:raise Utils.WafError('cannot find %s'%filename)
		if self.type=='latex':
			task=self.create_task('latex')
			task.set_inputs(node)
			task.set_outputs(node.change_ext('.dvi'))
		elif self.type=='pdflatex':
			task=self.create_task('pdflatex')
			task.set_inputs(node)
			task.set_outputs(node.change_ext('.pdf'))
		task.env=self.env
		task.curdirnode=self.path
		if deps_lst:
			variant=node.variant(self.env)
			try:
				lst=tree.node_deps[task.unique_id()]
				for n in deps_lst:
					if not n in lst:
						lst.append(n)
			except KeyError:
				tree.node_deps[task.unique_id()]=deps_lst
		if self.type=='latex':
			if'ps'in outs:
				pstask=self.create_task('dvips')
				pstask.set_inputs(task.outputs)
				pstask.set_outputs(node.change_ext('.ps'))
			if'pdf'in outs:
				pdftask=self.create_task('dvipdf')
				pdftask.set_inputs(task.outputs)
				pdftask.set_outputs(node.change_ext('.pdf'))
		elif self.type=='pdflatex':
			if'ps'in outs:
				pstask=self.create_task('pdf2ps')
				pstask.set_inputs(task.outputs)
				pstask.set_outputs(node.change_ext('.ps'))
	self.source=[]
def detect(conf):
	v=conf.env
	for p in'tex latex pdflatex bibtex dvips dvipdf ps2pdf makeindex pdf2ps'.split():
		conf.find_program(p,var=p.upper())
		v[p.upper()+'FLAGS']=''
	v['DVIPSFLAGS']='-Ppdf'
b=Task.simple_task_type
b('tex','${TEX} ${TEXFLAGS} ${SRC}',color='BLUE',shell=False)
b('bibtex','${BIBTEX} ${BIBTEXFLAGS} ${SRC}',color='BLUE',shell=False)
b('dvips','${DVIPS} ${DVIPSFLAGS} ${SRC} -o ${TGT}',color='BLUE',after="latex pdflatex tex bibtex",shell=False)
b('dvipdf','${DVIPDF} ${DVIPDFFLAGS} ${SRC} ${TGT}',color='BLUE',after="latex pdflatex tex bibtex",shell=False)
b('pdf2ps','${PDF2PS} ${PDF2PSFLAGS} ${SRC} ${TGT}',color='BLUE',after="dvipdf pdflatex",shell=False)
b=Task.task_type_from_func
cls=b('latex',latex_build,vars=latex_vardeps)
cls.scan=scan
cls=b('pdflatex',pdflatex_build,vars=pdflatex_vardeps)
cls.scan=scan

feature('tex')(apply_tex)
before('apply_core')(apply_tex)



#! /usr/bin/env python
# encoding: utf-8

import sys
import ar
def find_gdc(conf):
	v=conf.env
	d_compiler=None
	if v['D_COMPILER']:
		d_compiler=v['D_COMPILER']
	if not d_compiler:d_compiler=conf.find_program('gdc',var='D_COMPILER')
	if not d_compiler:return 0
	v['D_COMPILER']=d_compiler
def common_flags(conf):
	v=conf.env
	v['DFLAGS']=[]
	v['D_SRC_F']=''
	v['D_TGT_F']=['-c','-o','']
	v['DPATH_ST']='-I%s'
	v['D_LINKER']=v['D_COMPILER']
	v['DLNK_SRC_F']=''
	v['DLNK_TGT_F']=['-o','']
	v['DLIB_ST']='-l%s'
	v['DLIBPATH_ST']='-L%s'
	v['DLINKFLAGS']=[]
	v['DFLAGS_OPTIMIZED']=['-O3']
	v['DFLAGS_DEBUG']=['-O0']
	v['DFLAGS_ULTRADEBUG']=['-O0']
	v['D_shlib_DFLAGS']=[]
	v['D_shlib_LINKFLAGS']=['-shared']
	v['DHEADER_ext']='.di'
	v['D_HDR_F']='-fintfc -fintfc-file='
	if sys.platform=="win32":
		v['D_program_PATTERN']='%s.exe'
		v['D_shlib_PATTERN']='lib%s.dll'
		v['D_staticlib_PATTERN']='lib%s.a'
	else:
		v['D_program_PATTERN']='%s'
		v['D_shlib_PATTERN']='lib%s.so'
		v['D_staticlib_PATTERN']='lib%s.a'
def detect(conf):
	v=conf.env
	find_gdc(conf)
	ar.find_ar(conf)
	conf.check_tool('d')
	common_flags(conf)




#! /usr/bin/env python
# encoding: utf-8

import os,sys
import Build,TaskGen,Utils,Options,Logs
class unit_test(object):
	def __init__(self):
		self.returncode_ok=0
		self.num_tests_ok=0
		self.num_tests_failed=0
		self.num_tests_err=0
		self.total_num_tests=0
		self.max_label_length=0
		self.unit_tests=Utils.ordered_dict()
		self.unit_test_results={}
		self.unit_test_erroneous={}
		self.change_to_testfile_dir=False
		self.want_to_see_test_output=False
		self.want_to_see_test_error=False
		self.run_if_waf_does='check'
	def run(self):
		self.num_tests_ok=0
		self.num_tests_failed=0
		self.num_tests_err=0
		self.total_num_tests=0
		self.max_label_length=0
		self.unit_tests=Utils.ordered_dict()
		self.unit_test_results={}
		self.unit_test_erroneous={}
		ld_library_path=[]
		if not Options.commands[self.run_if_waf_does]:return
		for obj in Build.bld.all_task_gen:
			try:
				link_task=obj.link_task
			except AttributeError:
				pass
			else:
				lib_path=link_task.outputs[0].parent.abspath(obj.env)
				if lib_path not in ld_library_path:
					ld_library_path.append(lib_path)
			unit_test=getattr(obj,'unit_test','')
			if unit_test and'cprogram'in obj.features:
				try:
					output=obj.path
					filename=os.path.join(output.abspath(obj.env),obj.target)
					srcdir=output.abspath()
					label=os.path.join(output.bldpath(obj.env),obj.target)
					self.max_label_length=max(self.max_label_length,len(label))
					self.unit_tests[label]=(filename,srcdir)
				except KeyError:
					pass
		self.total_num_tests=len(self.unit_tests)
		Utils.pprint('GREEN','Running the unit tests')
		count=0
		result=1
		for label in self.unit_tests.allkeys:
			file_and_src=self.unit_tests[label]
			filename=file_and_src[0]
			srcdir=file_and_src[1]
			count+=1
			line=Build.bld.progress_line(count,self.total_num_tests,Logs.colors.GREEN,Logs.colors.NORMAL)
			if Options.options.progress_bar and line:
				sys.stderr.write(line)
				sys.stderr.flush()
			try:
				kwargs={}
				kwargs['env']=os.environ.copy()
				if self.change_to_testfile_dir:
					kwargs['cwd']=srcdir
				if not self.want_to_see_test_output:
					kwargs['stdout']=Utils.pproc.PIPE
				if not self.want_to_see_test_error:
					kwargs['stderr']=Utils.pproc.PIPE
				if ld_library_path:
					if sys.platform=='win32':
						kwargs['env']['PATH']=';'.join(ld_library_path+[os.environ.get('PATH','')])
					else:
						kwargs['env']['LD_LIBRARY_PATH']=':'.join(ld_library_path+[os.environ.get('LD_LIBRARY_PATH','')])
				pp=Utils.pproc.Popen(filename,**kwargs)
				pp.wait()
				result=int(pp.returncode==self.returncode_ok)
				if result:
					self.num_tests_ok+=1
				else:
					self.num_tests_failed+=1
				self.unit_test_results[label]=result
				self.unit_test_erroneous[label]=0
			except OSError:
				self.unit_test_erroneous[label]=1
				self.num_tests_err+=1
			except KeyboardInterrupt:
				pass
		if Options.options.progress_bar:sys.stdout.write(Logs.colors.cursor_on)
	def print_results(self):
		if not Options.commands[self.run_if_waf_does]:return
		p=Utils.pprint
		if self.total_num_tests==0:
			p('YELLOW','No unit tests present')
			return
		for label in self.unit_tests.allkeys:
			filename=self.unit_tests[label]
			err=0
			result=0
			try:err=self.unit_test_erroneous[label]
			except KeyError:pass
			try:result=self.unit_test_results[label]
			except KeyError:pass
			n=self.max_label_length-len(label)
			if err:n+=4
			elif result:n+=7
			else:n+=3
			line='%s %s'%(label,'.'*n)
			if err:p('RED','%sERROR'%line)
			elif result:p('GREEN','%sOK'%line)
			else:p('YELLOW','%sFAILED'%line)
		percentage_ok=float(self.num_tests_ok)/float(self.total_num_tests)*100.0
		percentage_failed=float(self.num_tests_failed)/float(self.total_num_tests)*100.0
		percentage_erroneous=float(self.num_tests_err)/float(self.total_num_tests)*100.0
		p('NORMAL','''
Successful tests:      %i (%.1f%%)
Failed tests:          %i (%.1f%%)
Erroneous tests:       %i (%.1f%%)

Total number of tests: %i
'''%(self.num_tests_ok,percentage_ok,self.num_tests_failed,percentage_failed,self.num_tests_err,percentage_erroneous,self.total_num_tests))
		p('GREEN','Unit tests finished')




# -*- coding: utf-8 -*-
'''A waf tool for building Sphinx documentation.

.. moduleauthor:: Mark Lee <waf@lazymalevolence.com>
'''

import Constants
import Node
import os
import pproc
import stat
import Task
import TaskGen
import Utils


class sphinx_task(Task.Task):
    '''The task that builds Sphinx documentation.'''

    def update_build_dir(self, node):
        '''Adapted from Waf 1.5.15 (wafadmin/Node.py).'''

        path = node.abspath(self.env)

        lst = Utils.listdir(path)
        try:
            node.__class__.bld.cache_dir_contents[node.id].update(lst)
        except KeyError:
            node.__class__.bld.cache_dir_contents[node.id] = set(lst)
        node.__class__.bld.cache_scanned_folders[node.id] = True

        for k in lst:
            npath = path + os.sep + k
            st = os.stat(npath)
            if stat.S_ISREG(st[stat.ST_MODE]):
                try:
                    ick = node.find_or_declare(k)
                    if ick.id not in node.__class__.bld \
                                         .node_sigs[self.env.variant()]:
                        node.__class__.bld \
                            .node_sigs[self.env.variant()][ick.id] = \
                                Constants.SIG_NIL
                except AttributeError:
                    print 'WTF @ %s' % k
            elif stat.S_ISDIR(st[stat.ST_MODE]):
                child = node.find_dir(k)
                if not child:
                    child = node.ensure_dir_node_from_path(k)
                self.update_build_dir(child)

    def run(self):
        doc_dir = self.inputs[0].parent
        rule = '"${SPHINX}" -b html "%s" "%s"' % (doc_dir.srcpath(),
                                                  doc_dir.bldpath(self.env))
        cmd = Utils.subst_vars(rule, self.env)
        proc = pproc.Popen(cmd, shell=True, stdin=pproc.PIPE)
        proc.communicate()
        self.update_build_dir(self.generator.path)
        return proc.returncode

    def install(self):
        base_builddir = self.inputs[0].parent.bldpath(self.env)
        exclude = Node.exclude_regs + '\n**/.buildinfo\n**/_static'

        def generate_glob(pattern, use_exclude=False):
            excl = []
            if use_exclude:
                excl = exclude
            glob = self.generator.path.ant_glob(pattern, dir=False, src=False,
                                                excl=excl)
            return ' '.join([os.path.join('build', base_builddir, x)
                             for x in glob.split()])
        glob_base = generate_glob('*', True)
        glob_static = generate_glob('_static/*')
        glob_sources = ' '.join([os.path.join('docs', x)
                                 for x in self.generator.path \
                                              .ant_glob('*.rst').split()])
        self.generator.bld.install_files('${HTMLDIR}', glob_base)
        self.generator.bld.install_files('${HTMLDIR}/_static', glob_static)
        for rst in glob_sources.split():
            basename = os.path.splitext(os.path.basename(rst))[0]
            self.generator.bld.install_as('${HTMLDIR}/_sources/%s.txt' % \
                                          basename, rst)


@TaskGen.extension('.rst')
def rst_handler(task, node):
    # do nothing!
    pass


@TaskGen.feature('sphinx')
def process_sphinx(self):
    conf_file = getattr(self, 'sphinx_config', 'conf.py')

    node = self.path.find_resource(conf_file)
    if not node:
        raise ValueError('sphinx configuration file not found')

    # the task instance
    task = self.create_task('sphinx')
    source = self.source
    self.source = ['%s.rst' % s for s in source]
    self.target = ['objects.inv', 'search.html', 'searchindex.js'] + \
                  ['%s.html' % s for s in source]
    task.set_inputs([node] + [self.path.find_resource(f)
                              for f in self.source])
    task.set_outputs([self.path.find_or_declare(f) for f in self.target])


def detect(conf):
    conf.find_program('sphinx-build', mandatory=True, var='SPHINX')



#! /usr/bin/env python
# encoding: utf-8

import os
import TaskGen,Task,Utils
from TaskGen import taskgen,before,extension
nasm_str='${NASM} ${NASM_FLAGS} ${NASM_INCLUDES} ${SRC} -o ${TGT}'
EXT_NASM=['.s','.S','.asm','.ASM','.spp','.SPP']
def apply_nasm_vars(self):
	if hasattr(self,'nasm_flags'):
		for flag in self.to_list(self.nasm_flags):
			self.env.append_value('NASM_FLAGS',flag)
	if hasattr(self,'includes'):
		for inc in self.to_list(self.includes):
			node=self.path.find_dir(inc)
			if not node:
				raise Utils.WafError('cannot find the dir'+inc)
			self.env.append_value('NASM_INCLUDES','-I%s'%node.srcpath(self.env))
			self.env.append_value('NASM_INCLUDES','-I%s'%node.bldpath(self.env))
def nasm_file(self,node):
	try:obj_ext=self.obj_ext
	except AttributeError:obj_ext='_%d.o'%self.idx
	task=self.create_task('nasm')
	task.inputs=[node]
	task.outputs=[node.change_ext(obj_ext)]
	self.compiled_tasks.append(task)
	self.meths.append('apply_nasm_vars')
Task.simple_task_type('nasm',nasm_str,color='BLUE',ext_out='.o',shell=False)
def detect(conf):
	nasm=conf.find_program('nasm',var='NASM')
	if not nasm:nasm=conf.find_program('yasm',var='NASM')
	if not nasm:conf.fatal('could not find nasm (or yasm), install it or set PATH env var')

before('apply_link')(apply_nasm_vars)
extension(EXT_NASM)(nasm_file)



#! /usr/bin/env python
# encoding: utf-8

import os,shutil,sys,platform
import TaskGen,Task,Build,Options
from TaskGen import taskgen,feature,after,before
from Logs import error,debug
def set_macosx_deployment_target(self):
	if self.env['MACOSX_DEPLOYMENT_TARGET']:
		os.environ['MACOSX_DEPLOYMENT_TARGET']=self.env['MACOSX_DEPLOYMENT_TARGET']
	elif'MACOSX_DEPLOYMENT_TARGET'not in os.environ:
		if sys.platform=='darwin':
			os.environ['MACOSX_DEPLOYMENT_TARGET']='.'.join(platform.mac_ver()[0].split('.')[:2])
def apply_framework(self):
	for x in self.to_list(self.env['FRAMEWORKPATH']):
		frameworkpath_st='-F%s'
		self.env.append_unique('CXXFLAGS',frameworkpath_st%x)
		self.env.append_unique('CCFLAGS',frameworkpath_st%x)
		self.env.append_unique('LINKFLAGS',frameworkpath_st%x)
	for x in self.to_list(self.env['FRAMEWORK']):
		self.env.append_value('LINKFLAGS',['-framework',x])
def create_task_macapp(self):
	if'cprogram'in self.features and self.link_task:
		apptask=self.create_task('macapp',self.env)
		apptask.set_inputs(self.link_task.outputs)
		apptask.set_outputs(self.link_task.outputs[0].change_ext('.app'))
		self.apptask=apptask
def apply_link_osx(self):
	if self.env['MACAPP']or getattr(self,'mac_app',False):
		self.create_task_macapp()
		name=self.link_task.outputs[0].name
		if getattr(self,'vnum',None):
			name=name.replace('.dylib','.%s.dylib'%self.vnum)
		path=os.path.join(self.env['PREFIX'],'lib',name)
		self.env.append_value('LINKFLAGS','-install_name')
		self.env.append_value('LINKFLAGS',path)
def apply_bundle(self):
	if not('cshlib'in self.features or'shlib'in self.features):return
	if self.env['MACBUNDLE']or getattr(self,'mac_bundle',False):
		self.env['shlib_PATTERN']=self.env['macbundle_PATTERN']
		uselib=self.uselib=self.to_list(self.uselib)
		if not'MACBUNDLE'in uselib:uselib.append('MACBUNDLE')
def apply_bundle_remove_dynamiclib(self):
	if self.env['MACBUNDLE']or getattr(self,'mac_bundle',False):
		if not getattr(self,'vnum',None):
			try:
				self.env['LINKFLAGS'].remove('-dynamiclib')
			except ValueError:
				pass
app_dirs=['Contents',os.path.join('Contents','MacOS'),os.path.join('Contents','Resources')]
app_info='''
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE plist SYSTEM "file://localhost/System/Library/DTDs/PropertyList.dtd">
<plist version="0.9">
<dict>
	<key>CFBundlePackageType</key>
	<string>APPL</string>
	<key>CFBundleGetInfoString</key>
	<string>Created by Waf</string>
	<key>CFBundleSignature</key>
	<string>????</string>
	<key>NOTE</key>
	<string>THIS IS A GENERATED FILE, DO NOT MODIFY</string>
	<key>CFBundleExecutable</key>
	<string>%s</string>
</dict>
</plist>
'''
def app_build(task):
	global app_dirs
	env=task.env
	i=0
	for p in task.outputs:
		srcfile=p.srcpath(env)
		debug('osx: creating directories')
		try:
			os.mkdir(srcfile)
			[os.makedirs(os.path.join(srcfile,d))for d in app_dirs]
		except(OSError,IOError):
			pass
		srcprg=task.inputs[i].srcpath(env)
		dst=os.path.join(srcfile,'Contents','MacOS')
		debug('osx: copy %s to %s'%(srcprg,dst))
		shutil.copy(srcprg,dst)
		debug('osx: generate Info.plist')
		f=open(os.path.join(srcfile,"Contents","Info.plist"),"w")
		f.write(app_info%os.path.basename(srcprg))
		f.close()
		i+=1
	return 0
def install_shlib(task):
	nums=task.vnum.split('.')
	path=self.install_path
	libname=task.outputs[0].name
	name3=libname.replace('.dylib','.%s.dylib'%task.vnum)
	name2=libname.replace('.dylib','.%s.dylib'%nums[0])
	name1=libname
	filename=task.outputs[0].abspath(task.env)
	bld=task.outputs[0].__class__.bld
	bld.install_as(path+name3,filename,env=task.env)
	bld.symlink_as(path+name2,name3)
	bld.symlink_as(path+name1,name3)
def install_target_osx_cshlib(self):
	if not self.bld.is_install:return
	if getattr(self,'vnum','')and sys.platform!='win32':
		self.link_task.install=install_shlib
Task.task_type_from_func('macapp',vars=[],func=app_build,after="cxx_link cc_link ar_link_static")

feature('cc','cxx')(set_macosx_deployment_target)
before('apply_lib_vars')(set_macosx_deployment_target)
feature('cc','cxx')(apply_framework)
after('apply_lib_vars')(apply_framework)
taskgen(create_task_macapp)
after('apply_link')(apply_link_osx)
feature('cc','cxx')(apply_link_osx)
before('apply_link','apply_lib_vars')(apply_bundle)
feature('cc','cxx')(apply_bundle)
after('apply_link')(apply_bundle_remove_dynamiclib)
feature('cshlib')(apply_bundle_remove_dynamiclib)
feature('osx')(install_target_osx_cshlib)
after('install_target_cshlib')(install_target_osx_cshlib)



#! /usr/bin/env python
# encoding: utf-8

import os,sys
import Configure,Options,Utils
import ccroot,ar,gcc
from Configure import conftest
def find_icc(conf):
	if sys.platform=='cygwin':
		conf.fatal('The Intel compiler does not work on Cygwin')
	v=conf.env
	cc=None
	if v['CC']:cc=v['CC']
	elif'CC'in conf.environ:cc=conf.environ['CC']
	if not cc:cc=conf.find_program('icc',var='CC')
	if not cc:cc=conf.find_program('ICL',var='CC')
	if not cc:conf.fatal('Intel C Compiler (icc) was not found')
	cc=conf.cmd_to_list(cc)
	ccroot.get_cc_version(conf,cc,icc=True)
	v['CC']=cc
	v['CC_NAME']='icc'
detect='''
find_icc
find_ar
gcc_common_flags
gcc_modifier_darwin
cc_load_tools
cc_add_flags
'''

conftest(find_icc)



#! /usr/bin/env python
# encoding: utf-8

import os,re
import TaskGen,Utils,Task,Build
from Logs import error
from TaskGen import taskgen,feature,before,after,extension
EXT_MLL=['.mll']
EXT_MLY=['.mly']
EXT_MLI=['.mli']
EXT_MLC=['.c']
EXT_ML=['.ml']
open_re=re.compile('^\s*open\s+([a-zA-Z]+)(;;){0,1}$',re.M)
foo=re.compile(r"""(\(\*)|(\*\))|("(\\.|[^"\\])*"|'(\\.|[^'\\])*'|.[^()*"'\\]*)""",re.M)
def filter_comments(txt):
	meh=[0]
	def repl(m):
		if m.group(1):meh[0]+=1
		elif m.group(2):meh[0]-=1
		elif not meh[0]:return m.group(0)
		return''
	return foo.sub(repl,txt)
def scan(self):
	node=self.inputs[0]
	code=filter_comments(node.read(self.env))
	global open_re
	names=[]
	import_iterator=open_re.finditer(code)
	if import_iterator:
		for import_match in import_iterator:
			names.append(import_match.group(1))
	found_lst=[]
	raw_lst=[]
	for name in names:
		nd=None
		for x in self.incpaths:
			nd=x.find_resource(name.lower()+'.ml')
			if not nd:nd=x.find_resource(name+'.ml')
			if nd:
				found_lst.append(nd)
				break
		else:
			raw_lst.append(name)
	return(found_lst,raw_lst)
native_lst=['native','all','c_object']
bytecode_lst=['bytecode','all']
class ocaml_taskgen(TaskGen.task_gen):
	def __init__(self,*k,**kw):
		TaskGen.task_gen.__init__(self,*k,**kw)
def init_ml(self):
	Utils.def_attrs(self,type='all',incpaths_lst=[],bld_incpaths_lst=[],mlltasks=[],mlytasks=[],mlitasks=[],native_tasks=[],bytecode_tasks=[],linktasks=[],bytecode_env=None,native_env=None,compiled_tasks=[],includes='',uselib='',are_deps_set=0)
def init_envs_ml(self):
	self.islibrary=getattr(self,'islibrary',False)
	global native_lst,bytecode_lst
	self.native_env=None
	if self.type in native_lst:
		self.native_env=self.env.copy()
		if self.islibrary:self.native_env['OCALINKFLAGS']='-a'
	self.bytecode_env=None
	if self.type in bytecode_lst:
		self.bytecode_env=self.env.copy()
		if self.islibrary:self.bytecode_env['OCALINKFLAGS']='-a'
	if self.type=='c_object':
		self.native_env.append_unique('OCALINKFLAGS_OPT','-output-obj')
def apply_incpaths_ml(self):
	inc_lst=self.includes.split()
	lst=self.incpaths_lst
	for dir in inc_lst:
		node=self.path.find_dir(dir)
		if not node:
			error("node not found: "+str(dir))
			continue
		self.bld.rescan(node)
		if not node in lst:lst.append(node)
		self.bld_incpaths_lst.append(node)
def apply_vars_ml(self):
	for i in self.incpaths_lst:
		if self.bytecode_env:
			app=self.bytecode_env.append_value
			app('OCAMLPATH','-I')
			app('OCAMLPATH',i.srcpath(self.env))
			app('OCAMLPATH','-I')
			app('OCAMLPATH',i.bldpath(self.env))
		if self.native_env:
			app=self.native_env.append_value
			app('OCAMLPATH','-I')
			app('OCAMLPATH',i.bldpath(self.env))
			app('OCAMLPATH','-I')
			app('OCAMLPATH',i.srcpath(self.env))
	varnames=['INCLUDES','OCAMLFLAGS','OCALINKFLAGS','OCALINKFLAGS_OPT']
	for name in self.uselib.split():
		for vname in varnames:
			cnt=self.env[vname+'_'+name]
			if cnt:
				if self.bytecode_env:self.bytecode_env.append_value(vname,cnt)
				if self.native_env:self.native_env.append_value(vname,cnt)
def apply_link_ml(self):
	if self.bytecode_env:
		ext=self.islibrary and'.cma'or'.run'
		linktask=self.create_task('ocalink')
		linktask.bytecode=1
		linktask.set_outputs(self.path.find_or_declare(self.target+ext))
		linktask.obj=self
		linktask.env=self.bytecode_env
		self.linktasks.append(linktask)
	if self.native_env:
		if self.type=='c_object':ext='.o'
		elif self.islibrary:ext='.cmxa'
		else:ext=''
		linktask=self.create_task('ocalinkx')
		linktask.set_outputs(self.path.find_or_declare(self.target+ext))
		linktask.obj=self
		linktask.env=self.native_env
		self.linktasks.append(linktask)
		self.compiled_tasks.append(linktask)
def mll_hook(self,node):
	mll_task=self.create_task('ocamllex',self.native_env)
	mll_task.set_inputs(node)
	mll_task.set_outputs(node.change_ext('.ml'))
	self.mlltasks.append(mll_task)
	self.allnodes.append(mll_task.outputs[0])
def mly_hook(self,node):
	mly_task=self.create_task('ocamlyacc',self.native_env)
	mly_task.set_inputs(node)
	mly_task.set_outputs([node.change_ext('.ml'),node.change_ext('.mli')])
	self.mlytasks.append(mly_task)
	self.allnodes.append(mly_task.outputs[0])
	task=self.create_task('ocamlcmi',self.native_env)
	task.set_inputs(mly_task.outputs[1])
	task.set_outputs(mly_task.outputs[1].change_ext('.cmi'))
def mli_hook(self,node):
	task=self.create_task('ocamlcmi',self.native_env)
	task.set_inputs(node)
	task.set_outputs(node.change_ext('.cmi'))
	self.mlitasks.append(task)
def mlc_hook(self,node):
	task=self.create_task('ocamlcc',self.native_env)
	task.set_inputs(node)
	task.set_outputs(node.change_ext('.o'))
	self.compiled_tasks.append(task)
def ml_hook(self,node):
	if self.native_env:
		task=self.create_task('ocamlx',self.native_env)
		task.set_inputs(node)
		task.set_outputs(node.change_ext('.cmx'))
		task.obj=self
		task.incpaths=self.bld_incpaths_lst
		self.native_tasks.append(task)
	if self.bytecode_env:
		task=self.create_task('ocaml',self.bytecode_env)
		task.set_inputs(node)
		task.obj=self
		task.bytecode=1
		task.incpaths=self.bld_incpaths_lst
		task.set_outputs(node.change_ext('.cmo'))
		self.bytecode_tasks.append(task)
def compile_may_start(self):
	if not getattr(self,'flag_deps',''):
		self.flag_deps=1
		if getattr(self,'bytecode',''):alltasks=self.obj.bytecode_tasks
		else:alltasks=self.obj.native_tasks
		self.signature()
		tree=self.generator.bld
		env=self.env
		for node in self.inputs:
			lst=tree.node_deps[self.unique_id()]
			for depnode in lst:
				for t in alltasks:
					if t==self:continue
					if depnode in t.inputs:
						self.set_run_after(t)
		delattr(self,'cache_sig')
		self.signature()
	return Task.Task.runnable_status(self)
b=Task.simple_task_type
cls=b('ocamlx','${OCAMLOPT} ${OCAMLPATH} ${OCAMLFLAGS} ${INCLUDES} -c -o ${TGT} ${SRC}',color='GREEN',shell=False)
cls.runnable_status=compile_may_start
cls.scan=scan
b=Task.simple_task_type
cls=b('ocaml','${OCAMLC} ${OCAMLPATH} ${OCAMLFLAGS} ${INCLUDES} -c -o ${TGT} ${SRC}',color='GREEN',shell=False)
cls.runnable_status=compile_may_start
cls.scan=scan
b('ocamlcmi','${OCAMLC} ${OCAMLPATH} ${INCLUDES} -o ${TGT} -c ${SRC}',color='BLUE',before="ocaml ocamlcc ocamlx")
b('ocamlcc','cd ${TGT[0].bld_dir(env)} && ${OCAMLOPT} ${OCAMLFLAGS} ${OCAMLPATH} ${INCLUDES} -c ${SRC[0].abspath(env)}',color='GREEN')
b('ocamllex','${OCAMLLEX} ${SRC} -o ${TGT}',color='BLUE',before="ocamlcmi ocaml ocamlcc")
b('ocamlyacc','${OCAMLYACC} -b ${TGT[0].bld_base(env)} ${SRC}',color='BLUE',before="ocamlcmi ocaml ocamlcc")
def link_may_start(self):
	if not getattr(self,'order',''):
		if getattr(self,'bytecode',0):alltasks=self.obj.bytecode_tasks
		else:alltasks=self.obj.native_tasks
		seen=[]
		pendant=[]+alltasks
		while pendant:
			task=pendant.pop(0)
			if task in seen:continue
			for x in task.run_after:
				if not x in seen:
					pendant.append(task)
					break
			else:
				seen.append(task)
		self.inputs=[x.outputs[0]for x in seen]
		self.order=1
	return Task.Task.runnable_status(self)
act=b('ocalink','${OCAMLC} -o ${TGT} ${INCLUDES} ${OCALINKFLAGS} ${SRC}',color='YELLOW',after="ocaml ocamlcc")
act.runnable_status=link_may_start
act=b('ocalinkx','${OCAMLOPT} -o ${TGT} ${INCLUDES} ${OCALINKFLAGS_OPT} ${SRC}',color='YELLOW',after="ocamlx ocamlcc")
act.runnable_status=link_may_start
def detect(conf):
	opt=conf.find_program('ocamlopt',var='OCAMLOPT')
	occ=conf.find_program('ocamlc',var='OCAMLC')
	if(not opt)or(not occ):
		conf.fatal('The objective caml compiler was not found:\ninstall it or make it available in your PATH')
	conf.env['OCAMLC']=occ
	conf.env['OCAMLOPT']=opt
	conf.env['OCAMLLEX']=conf.find_program('ocamllex',var='OCAMLLEX')
	conf.env['OCAMLYACC']=conf.find_program('ocamlyacc',var='OCAMLYACC')
	conf.env['OCAMLFLAGS']=''
	conf.env['OCAMLLIB']=Utils.cmd_output(conf.env['OCAMLC']+' -where').strip()+os.sep
	conf.env['LIBPATH_OCAML']=Utils.cmd_output(conf.env['OCAMLC']+' -where').strip()+os.sep
	conf.env['CPPPATH_OCAML']=Utils.cmd_output(conf.env['OCAMLC']+' -where').strip()+os.sep
	conf.env['LIB_OCAML']='camlrun'

feature('ocaml')(init_ml)
feature('ocaml')(init_envs_ml)
after('init_ml')(init_envs_ml)
feature('ocaml')(apply_incpaths_ml)
before('apply_vars_ml')(apply_incpaths_ml)
after('init_envs_ml')(apply_incpaths_ml)
feature('ocaml')(apply_vars_ml)
before('apply_core')(apply_vars_ml)
feature('ocaml')(apply_link_ml)
after('apply_core')(apply_link_ml)
extension(EXT_MLL)(mll_hook)
extension(EXT_MLY)(mly_hook)
extension(EXT_MLI)(mli_hook)
extension(EXT_MLC)(mlc_hook)
extension(EXT_ML)(ml_hook)



#! /usr/bin/env python
# encoding: utf-8

import os
import Task,Options,Utils
from Configure import conf
from TaskGen import extension,taskgen,feature,before
xsubpp_str='${PERL} ${XSUBPP} -noprototypes -typemap ${EXTUTILS_TYPEMAP} ${SRC} > ${TGT}'
EXT_XS=['.xs']
def init_perlext(self):
	self.uselib=self.to_list(getattr(self,'uselib',''))
	if not'PERL'in self.uselib:self.uselib.append('PERL')
	if not'PERLEXT'in self.uselib:self.uselib.append('PERLEXT')
	self.env['shlib_PATTERN']=self.env['perlext_PATTERN']
def xsubpp_file(self,node):
	gentask=self.create_task('xsubpp')
	gentask.set_inputs(node)
	outnode=node.change_ext('.c')
	gentask.set_outputs(outnode)
	self.allnodes.append(outnode)
Task.simple_task_type('xsubpp',xsubpp_str,color='BLUE',before="cc cxx",shell=False)
def check_perl_version(conf,minver=None):
	res=True
	if not getattr(Options.options,'perlbinary',None):
		perl=conf.find_program("perl",var="PERL")
		if not perl:
			return False
	else:
		perl=Options.options.perlbinary
		conf.env['PERL']=perl
	version=Utils.cmd_output(perl+" -e'printf \"%vd\", $^V'")
	if not version:
		res=False
		version="Unknown"
	elif not minver is None:
		ver=tuple(map(int,version.split(".")))
		if ver<minver:
			res=False
	if minver is None:
		cver=""
	else:
		cver=".".join(map(str,minver))
	conf.check_message("perl",cver,res,version)
	return res
def check_perl_module(conf,module):
	cmd=[conf.env['PERL'],'-e','use %s'%module]
	r=Utils.pproc.call(cmd,stdout=Utils.pproc.PIPE,stderr=Utils.pproc.PIPE)==0
	conf.check_message("perl module %s"%module,"",r)
	return r
def check_perl_ext_devel(conf):
	if not conf.env['PERL']:
		return False
	perl=conf.env['PERL']
	def read_out(cmd):
		return Utils.to_list(Utils.cmd_output(perl+cmd))
	conf.env["LINKFLAGS_PERLEXT"]=read_out(" -MConfig -e'print $Config{lddlflags}'")
	conf.env["CPPPATH_PERLEXT"]=read_out(" -MConfig -e'print \"$Config{archlib}/CORE\"'")
	conf.env["CCFLAGS_PERLEXT"]=read_out(" -MConfig -e'print \"$Config{ccflags} $Config{cccdlflags}\"'")
	conf.env["XSUBPP"]=read_out(" -MConfig -e'print \"$Config{privlib}/ExtUtils/xsubpp$Config{exe_ext}\"'")
	conf.env["EXTUTILS_TYPEMAP"]=read_out(" -MConfig -e'print \"$Config{privlib}/ExtUtils/typemap\"'")
	if not getattr(Options.options,'perlarchdir',None):
		conf.env["ARCHDIR_PERL"]=Utils.cmd_output(perl+" -MConfig -e'print $Config{sitearch}'")
	else:
		conf.env["ARCHDIR_PERL"]=getattr(Options.options,'perlarchdir')
	conf.env['perlext_PATTERN']='%s.'+Utils.cmd_output(perl+" -MConfig -e'print $Config{dlext}'")
	return True
def set_options(opt):
	opt.add_option("--with-perl-binary",type="string",dest="perlbinary",help='Specify alternate perl binary',default=None)
	opt.add_option("--with-perl-archdir",type="string",dest="perlarchdir",help='Specify directory where to install arch specific files',default=None)

before('apply_incpaths','apply_type_vars','apply_lib_vars')(init_perlext)
feature('perlext')(init_perlext)
extension(EXT_XS)(xsubpp_file)
conf(check_perl_version)
conf(check_perl_module)
conf(check_perl_ext_devel)



#! /usr/bin/env python
# encoding: utf-8

import os,sys,imp,types,ccroot
import optparse
import Utils,Configure,Options
from Logs import debug
cxx_compiler={'win32':['msvc','g++'],'cygwin':['g++'],'darwin':['g++'],'aix5':['g++'],'linux':['g++','icpc','sunc++'],'sunos':['g++','sunc++'],'irix':['g++'],'hpux':['g++'],'default':['g++']}
def __list_possible_compiler(platform):
	try:
		return(cxx_compiler[platform])
	except KeyError:
		return(cxx_compiler["default"])
def detect(conf):
	try:test_for_compiler=Options.options.check_cxx_compiler
	except AttributeError:raise Configure.ConfigurationError("Add set_options(opt): opt.tool_options('compiler_cxx')")
	for compiler in test_for_compiler.split():
		try:
			conf.check_tool(compiler)
		except Configure.ConfigurationError,e:
			debug('compiler_cxx: %r'%e)
		else:
			if conf.env['CXX']:
				conf.check_message(compiler,'',True)
				conf.env['COMPILER_CXX']=compiler
				break
			conf.check_message(compiler,'',False)
def set_options(opt):
	detected_platform=Options.platform
	possible_compiler_list=__list_possible_compiler(detected_platform)
	test_for_compiler=str(" ").join(possible_compiler_list)
	cxx_compiler_opts=opt.add_option_group("C++ Compiler Options")
	cxx_compiler_opts.add_option('--check-cxx-compiler',default="%s"%test_for_compiler,help='On this platform (%s) the following C++ Compiler will be checked by default: "%s"'%(detected_platform,test_for_compiler),dest="check_cxx_compiler")
	for cxx_compiler in test_for_compiler.split():
		opt.tool_options('%s'%cxx_compiler,option_group=cxx_compiler_opts)




#!/usr/bin/env python
#
# Copyright (c) 2009 Mark Lee <libdesktop-agnostic@lazymalevolence.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.

import desktopagnostic
from gtk import gdk

green = desktopagnostic.Color.from_string('green')
print green.to_string()
print green
assert isinstance(green.props.color, gdk.Color)
print str(green.props.color)
one_char_hex = desktopagnostic.Color.from_string('#f00f')
print one_char_hex.to_string()
print one_char_hex



#! /usr/bin/env python
# encoding: utf-8

import os,sys
import Task
from TaskGen import extension,taskgen,after,before
EXT_ASM=['.s','.S','.asm','.ASM','.spp','.SPP']
as_str='${AS} ${ASFLAGS} ${_ASINCFLAGS} ${SRC} -o ${TGT}'
Task.simple_task_type('asm',as_str,'PINK',ext_out='.o',shell=False)
def asm_hook(self,node):
	task=self.create_task('asm')
	try:obj_ext=self.obj_ext
	except AttributeError:obj_ext='_%d.o'%self.idx
	task.inputs=[node]
	task.outputs=[node.change_ext(obj_ext)]
	self.compiled_tasks.append(task)
	self.meths.append('asm_incflags')
def asm_incflags(self):
	if self.env['ASINCFLAGS']:self.env['_ASINCFLAGS']=self.env['ASINCFLAGS']
	if'cxx'in self.features:self.env['_ASINCFLAGS']=self.env['_CXXINCFLAGS']
	else:self.env['_ASINCFLAGS']=self.env['_CCINCFLAGS']
def detect(conf):
	comp=conf.environ.get('AS','')
	if not comp:comp=conf.env['AS']
	if not comp:comp=conf.find_program('as',var='AS')
	if not comp:comp=conf.find_program('gas',var='AS')
	if not comp:comp=conf.env['CC']
	if not comp:return
	v=conf.env
	v['ASFLAGS']=''

extension(EXT_ASM)(asm_hook)
taskgen(asm_incflags)
after('apply_obj_vars_cc')(asm_incflags)
after('apply_obj_vars_cxx')(asm_incflags)
before('apply_link')(asm_incflags)



#!/usr/bin/env python
#
# Copyright (c) 2009 Mark Lee <libdesktop-agnostic@lazymalevolence.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.

import sys
import os.path
import gobject
from desktopagnostic import vfs


class TestFileMonitor:

    def __init__(self, path):
        self.vfile = vfs.File.for_path(path)
        self.monitor = self.vfile.monitor()
        self.monitor.connect('changed', self.on_change)
        if self.vfile.props.file_type == vfs.FILE_TYPE_DIRECTORY:
            gobject.timeout_add_seconds(2, self.do_emit)

    def on_change(self, monitor, vfile, other, event):
        print '%s: %s' % (event.value_nick, vfile.props.uri)
        if other is not None:
            print ' * other: %s' % other.props.uri

    def do_emit(self):
        path = os.path.join(self.vfile.props.path, 'test-vfs-file.txt')
        other = vfs.File.for_path(path)
        self.monitor.changed(other, vfs.FILE_MONITOR_EVENT_CREATED)


def main(args):
    if len(args) < 2:
        sys.stderr.write('Usage: %s [FILE | DIRECTORY FILE] \n' % args[0])
        return 1
    vfs.init()
    try:
        ml = gobject.MainLoop()
        test = TestFileMonitor(args[1])
        ml.run()
        test.monitor.cancel()
        assert test.monitor.props.cancelled
    finally:
        vfs.shutdown()
    return 0

if __name__ == '__main__':
    sys.exit(main(sys.argv))



#! /usr/bin/env python
# encoding: utf-8

import os,sys,re,optparse
import ccroot
import TaskGen,Utils,Task,Configure,Logs,Build
from Logs import debug,error
from TaskGen import taskgen,feature,after,before,extension
EXT_D=['.d','.di','.D']
D_METHS=['apply_core','apply_vnum','apply_objdeps']
def filter_comments(filename):
	txt=Utils.readf(filename)
	buf=[]
	i=0
	max=len(txt)
	while i<max:
		c=txt[i]
		if c=='"':
			i+=1
			c=''
			while i<max:
				p=c
				c=txt[i]
				i+=1
				if i==max:return buf
				if c=='"':
					cnt=0
					while i<cnt and i<max:
						if txt[i-2-cnt]=='\\':cnt+=1
						else:break
					if(cnt%2)==0:break
		elif c=="'":
			i+=1
			if i==max:return buf
			c=txt[i]
			if c=='\\':
				i+=1
				if i==max:return buf
				c=txt[i]
				if c=='x':
					i+=2
				elif c=='u':
					i+=4
			i+=1
			if i==max:return buf
			c=txt[i]
			if c!='\'':error("uh-oh, invalid character")
		elif c=='/':
			if i==max:break
			c=txt[i+1]
			if c=='+':
				i+=1
				nesting=1
				prev=0
				while i<max:
					c=txt[i]
					if c=='+':
						prev=1
					elif c=='/':
						if prev:
							nesting-=1
							if nesting==0:break
						else:
							if i<max:
								i+=1
								c=txt[i]
								if c=='+':
									nesting+=1
							else:
								return buf
					else:
						prev=0
					i+=1
			elif c=='*':
				i+=1
				while i<max:
					c=txt[i]
					if c=='*':
						prev=1
					elif c=='/':
						if prev:break
					else:
						prev=0
					i+=1
			elif c=='/':
				i+=1
				c=txt[i]
				while i<max and c!='\n':
					i+=1
					c=txt[i]
		else:
			buf.append(c)
		i+=1
	return buf
class d_parser(object):
	def __init__(self,env,incpaths):
		self.allnames=[]
		self.re_module=re.compile("module\s+([^;]+)")
		self.re_import=re.compile("import\s+([^;]+)")
		self.re_import_bindings=re.compile("([^:]+):(.*)")
		self.re_import_alias=re.compile("[^=]+=(.+)")
		self.env=env
		self.nodes=[]
		self.names=[]
		self.incpaths=incpaths
	def tryfind(self,filename):
		found=0
		for n in self.incpaths:
			found=n.find_resource(filename.replace('.','/')+'.d')
			if found:
				self.nodes.append(found)
				self.waiting.append(found)
				break
		if not found:
			if not filename in self.names:
				self.names.append(filename)
	def get_strings(self,code):
		self.module=''
		lst=[]
		mod_name=self.re_module.search(code)
		if mod_name:
			self.module=re.sub('\s+','',mod_name.group(1))
		import_iterator=self.re_import.finditer(code)
		if import_iterator:
			for import_match in import_iterator:
				import_match_str=re.sub('\s+','',import_match.group(1))
				bindings_match=self.re_import_bindings.match(import_match_str)
				if bindings_match:
					import_match_str=bindings_match.group(1)
				matches=import_match_str.split(',')
				for match in matches:
					alias_match=self.re_import_alias.match(match)
					if alias_match:
						match=alias_match.group(1)
					lst.append(match)
		return lst
	def start(self,node):
		self.waiting=[node]
		while self.waiting:
			nd=self.waiting.pop(0)
			self.iter(nd)
	def iter(self,node):
		path=node.abspath(self.env)
		code="".join(filter_comments(path))
		names=self.get_strings(code)
		for x in names:
			if x in self.allnames:continue
			self.allnames.append(x)
			self.tryfind(x)
def scan(self):
	env=self.env
	gruik=d_parser(env,env['INC_PATHS'])
	gruik.start(self.inputs[0])
	if Logs.verbose:
		debug('deps: nodes found for %s: %s %s'%(str(self.inputs[0]),str(gruik.nodes),str(gruik.names)))
	return(gruik.nodes,gruik.names)
def get_target_name(self):
	v=self.env
	tp='program'
	for x in self.features:
		if x in['dshlib','dstaticlib']:
			tp=x.lstrip('d')
	return v['D_%s_PATTERN'%tp]%self.target
d_params={'dflags':'','importpaths':'','libs':'','libpaths':'','generate_headers':False,}
def init_d(self):
	for x in d_params:
		setattr(self,x,getattr(self,x,d_params[x]))
class d_taskgen(TaskGen.task_gen):
	def __init__(self,*k,**kw):
		TaskGen.task_gen.__init__(self,*k,**kw)
		if len(k)>1:
			self.features.append('d'+k[1])
TaskGen.bind_feature('d',D_METHS)
def init_d(self):
	Utils.def_attrs(self,dflags='',importpaths='',libs='',libpaths='',uselib='',uselib_local='',generate_headers=False,compiled_tasks=[],add_objects=[],link_task=None)
def apply_d_libs(self):
	uselib=self.to_list(self.uselib)
	seen=[]
	local_libs=self.to_list(self.uselib_local)
	libs=[]
	libpaths=[]
	env=self.env
	while local_libs:
		x=local_libs.pop()
		if x in seen:
			continue
		else:
			seen.append(x)
		y=self.name_to_obj(x)
		if not y:
			raise Utils.WafError('object not found in uselib_local: obj %s uselib %s'%(self.name,x))
		if y.uselib_local:
			added=0
			lst=y.to_list(y.uselib_local)
			lst.reverse()
			for u in lst:
				if u in seen:continue
				added=1
				local_libs=[u]+local_libs
			if added:continue
		y.post()
		seen.append(x)
		libname=y.target[y.target.rfind(os.sep)+1:]
		if'dshlib'in y.features or'dstaticlib'in y.features:
			env.append_unique('DLINKFLAGS',env['DLIBPATH_ST']%y.link_task.outputs[0].parent.bldpath(env))
			env.append_unique('DLINKFLAGS',env['DLIB_ST']%libname)
		tmp_path=y.path.bldpath(env)
		if not tmp_path in libpaths:libpaths=[tmp_path]+libpaths
		if y.link_task is not None:
			self.link_task.set_run_after(y.link_task)
			dep_nodes=getattr(self.link_task,'dep_nodes',[])
			self.link_task.dep_nodes=dep_nodes+y.link_task.outputs
		morelibs=y.to_list(y.uselib)
		for v in morelibs:
			if v in uselib:continue
			uselib=[v]+uselib
	self.uselib=uselib
def apply_d_link(self):
	link=getattr(self,'link',None)
	if not link:
		if'dstaticlib'in self.features:link='ar_link_static'
		else:link='d_link'
	linktask=self.create_task(link)
	outputs=[t.outputs[0]for t in self.compiled_tasks]
	linktask.set_inputs(outputs)
	linktask.set_outputs(self.path.find_or_declare(get_target_name(self)))
	self.link_task=linktask
def apply_d_vars(self):
	env=self.env
	dpath_st=env['DPATH_ST']
	lib_st=env['DLIB_ST']
	libpath_st=env['DLIBPATH_ST']
	importpaths=self.to_list(self.importpaths)
	libpaths=[]
	libs=[]
	uselib=self.to_list(self.uselib)
	for i in uselib:
		if env['DFLAGS_'+i]:
			env.append_unique('DFLAGS',env['DFLAGS_'+i])
	for x in self.features:
		if not x in['dprogram','dstaticlib','dshlib']:
			continue
		x.lstrip('d')
		d_shlib_dflags=env['D_'+x+'_DFLAGS']
		if d_shlib_dflags:
			env.append_unique('DFLAGS',d_shlib_dflags)
	for i in uselib:
		if env['DPATH_'+i]:
			for entry in self.to_list(env['DPATH_'+i]):
				if not entry in importpaths:
					importpaths.append(entry)
	for path in importpaths:
		if os.path.isabs(path):
			env.append_unique('_DIMPORTFLAGS',dpath_st%path)
		else:
			node=self.path.find_dir(path)
			self.env.append_unique('INC_PATHS',node)
			env.append_unique('_DIMPORTFLAGS',dpath_st%node.srcpath(env))
			env.append_unique('_DIMPORTFLAGS',dpath_st%node.bldpath(env))
	for i in uselib:
		if env['LIBPATH_'+i]:
			for entry in self.to_list(env['LIBPATH_'+i]):
				if not entry in libpaths:
					libpaths+=[entry]
	libpaths=self.to_list(self.libpaths)+libpaths
	for path in libpaths:
		env.append_unique('DLINKFLAGS',libpath_st%path)
	for i in uselib:
		if env['LIB_'+i]:
			for entry in self.to_list(env['LIB_'+i]):
				if not entry in libs:
					libs+=[entry]
	libs=libs+self.to_list(self.libs)
	for lib in libs:
		env.append_unique('DLINKFLAGS',lib_st%lib)
	for i in uselib:
		dlinkflags=env['DLINKFLAGS_'+i]
		if dlinkflags:
			for linkflag in dlinkflags:
				env.append_unique('DLINKFLAGS',linkflag)
def add_shlib_d_flags(self):
	for linkflag in self.env['D_shlib_LINKFLAGS']:
		self.env.append_unique('DLINKFLAGS',linkflag)
def d_hook(self,node):
	task=self.create_task(self.generate_headers and'd_with_header'or'd')
	try:obj_ext=self.obj_ext
	except AttributeError:obj_ext='_%d.o'%self.idx
	task.inputs=[node]
	task.outputs=[node.change_ext(obj_ext)]
	self.compiled_tasks.append(task)
	if self.generate_headers:
		header_node=node.change_ext(self.env['DHEADER_ext'])
		task.outputs+=[header_node]
d_str='${D_COMPILER} ${DFLAGS} ${_DIMPORTFLAGS} ${D_SRC_F}${SRC} ${D_TGT_F}${TGT}'
d_with_header_str='${D_COMPILER} ${DFLAGS} ${_DIMPORTFLAGS} \
${D_HDR_F}${TGT[1].bldpath(env)} \
${D_SRC_F}${SRC} \
${D_TGT_F}${TGT[0].bldpath(env)}'
link_str='${D_LINKER} ${DLNK_SRC_F}${SRC} ${DLNK_TGT_F}${TGT} ${DLINKFLAGS}'
def override_exec(cls):
	old_exec=cls.exec_command
	def exec_command(self,*k,**kw):
		if self.env['COMPILER_D']=='dmd'and isinstance(k[0],list):
			lst=k[0]
			for i in xrange(len(lst)):
				if lst[i]=='-of':
					del lst[i]
					lst[i]='-of'+lst[i]
					break
		return old_exec(self,*k,**kw)
	cls.exec_command=exec_command
cls=Task.simple_task_type('d',d_str,'GREEN',before='ar_link_static d_link',shell=False)
cls.scan=scan
override_exec(cls)
cls=Task.simple_task_type('d_with_header',d_with_header_str,'GREEN',before='ar_link_static d_link',shell=False)
override_exec(cls)
cls=Task.simple_task_type('d_link',link_str,color='YELLOW',shell=False)
override_exec(cls)
def generate_header(self,filename,install_path):
	if not hasattr(self,'header_lst'):self.header_lst=[]
	self.meths.append('process_header')
	self.header_lst.append([filename,install_path])
def process_header(self):
	env=self.env
	for i in getattr(self,'header_lst',[]):
		node=self.path.find_resource(i[0])
		if not node:
			raise Utils.WafError('file not found on d obj '+i[0])
		task=self.create_task('d_header')
		task.set_inputs(node)
		task.set_outputs(node.change_ext('.di'))
d_header_str='${D_COMPILER} ${D_HEADER} ${SRC}'
Task.simple_task_type('d_header',d_header_str,color='BLUE',shell=False)

feature('d')(init_d)
before('apply_type_vars')(init_d)
feature('d')(init_d)
before('apply_d_libs')(init_d)
feature('d')(apply_d_libs)
after('apply_d_link')(apply_d_libs)
before('apply_vnum')(apply_d_libs)
feature('dprogram','dshlib','dstaticlib')(apply_d_link)
after('apply_core')(apply_d_link)
feature('d')(apply_d_vars)
after('apply_core')(apply_d_vars)
feature('dshlib')(add_shlib_d_flags)
after('apply_d_vars')(add_shlib_d_flags)
extension(EXT_D)(d_hook)
taskgen(generate_header)
before('apply_core')(process_header)
